﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ErikEJ.SqlCeToolbox.ToolWindows;
using ErikEJ.SqlCeScripting;
using Microsoft.VisualStudio.Data.Interop;
using Microsoft.Win32;
using ErikEJ.SqlCeToolbox.Dialogs;
using System.Linq;
using System.Collections.Generic;

namespace ErikEJ.SqlCeToolbox.Commands
{
    public class ColumnMenuCommandsHandler : CommandHandlerBase
    {
        public ColumnMenuCommandsHandler(ExplorerToolWindow parent)
        {
            ParentWindow = parent;
        }

        public void ScriptAsCreate(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var menuInfo = menuItem.CommandParameter as MenuCommandParameters;

                if (menuInfo != null)
                {
                    try
                    {
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(menuInfo.DatabaseInfo))
                        {
                            var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, menuInfo.DatabaseInfo.DatabaseType);
                            List<Column> columns = repository.GetAllColumns();
                            var col = columns.Where(c => c.TableName == menuInfo.Description && c.ColumnName == menuInfo.Name).SingleOrDefault();
                            if (col == null)
                            {
                                MessageBox.Show("Could not find the column in the table, has it been dropped?");
                            }
                            else
                            {
                                generator.GenerateColumnAddScript(col);
                            }
                            OpenSqlEditorToolWindow(menuInfo, generator.GeneratedScript, false);
                            Helpers.DataConnectionHelper.LogUsage("ColumnScriptAsCreate");
                        }
                    }
                    catch (Exception ex)
                    {
                        Helpers.DataConnectionHelper.SendError(ex, menuInfo.DatabaseInfo.DatabaseType, false);
                    }
                }
            }
        }

        public void ScriptAsDrop(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var menuInfo = menuItem.CommandParameter as MenuCommandParameters;

                if (menuInfo != null)
                {
                    try
                    {
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(menuInfo.DatabaseInfo))
                        {
                            var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, menuInfo.DatabaseInfo.DatabaseType);
                            List<Column> columns = repository.GetAllColumns();
                            var col = columns.Where(c => c.TableName == menuInfo.Description && c.ColumnName == menuInfo.Name).SingleOrDefault();
                            if (col == null)
                            {
                                MessageBox.Show("Could not find the column in the table, has it been dropped?");
                            }
                            else
                            {
                                generator.GenerateColumnDropScript(col);
                            }
                            OpenSqlEditorToolWindow(menuInfo, generator.GeneratedScript, false);
                            Helpers.DataConnectionHelper.LogUsage("ColumnScriptAsDrop");
                        }
                    }
                    catch (Exception ex)
                    {
                        Helpers.DataConnectionHelper.SendError(ex, menuInfo.DatabaseInfo.DatabaseType, false);
                    }
                }
            }
        }

        public void ScriptAsAlter(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var menuInfo = menuItem.CommandParameter as MenuCommandParameters;

                if (menuInfo != null)
                {
                    try
                    {
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(menuInfo.DatabaseInfo))
                        {
                            var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, menuInfo.DatabaseInfo.DatabaseType);
                            List<Column> columns = repository.GetAllColumns();
                            var col = columns.Where(c => c.TableName == menuInfo.Description && c.ColumnName == menuInfo.Name).SingleOrDefault();
                            if (col == null)
                            {
                                MessageBox.Show("Could not find the column in the table, has it been dropped?");
                            }
                            else
                            {
                                generator.GenerateColumnAlterScript(col);
                            }
                            OpenSqlEditorToolWindow(menuInfo, generator.GeneratedScript, false);
                            Helpers.DataConnectionHelper.LogUsage("ColumnScriptAsDrop");
                        }
                    }
                    catch (Exception ex)
                    {
                        Helpers.DataConnectionHelper.SendError(ex, menuInfo.DatabaseInfo.DatabaseType, false);
                    }
                }
            }
        }

        public void AddDescription(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var menuInfo = menuItem.CommandParameter as MenuCommandParameters;
                if (menuInfo != null)
                {
                    string name = menuInfo.Description;
                    new BaseCommandHandler(ParentWindow).UpdateDescriptions(menuInfo, name);
                }
            }
        }
    }
}