﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data.EntityClient;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using EnvDTE;
using ErikEJ.SqlCeScripting;
using ErikEJ.SqlCeToolbox.Dialogs;
using ErikEJ.SqlCeToolbox.Helpers;
using ErikEJ.SqlCeToolbox.ToolWindows;
using Microsoft.VisualStudio.Data.Interop;
using Microsoft.Win32;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio;
using System.ComponentModel;

namespace ErikEJ.SqlCeToolbox.Commands
{
    public class DatabaseMenuCommandsHandler
    {
        private readonly ExplorerToolWindow _parentWindow;
        private SqlCeToolboxPackage package;

        public DatabaseMenuCommandsHandler(ExplorerToolWindow parent)
        {
            _parentWindow = parent;
            package = _parentWindow.Package as SqlCeToolboxPackage;
        }

        #region Root Level Commands
        public void AddPrivateCe35Database(object sender, ExecutedRoutedEventArgs e)
        {
            AddCeDatabase(DatabaseType.SQLCE35);
        }

        public void AddPrivateCe40Database(object sender, ExecutedRoutedEventArgs e)
        {
            AddCeDatabase(DatabaseType.SQLCE40);
        }

        private void AddCeDatabase(DatabaseType dbType)
        {
            try
            {
                var dialog = new ConnectionDialog();
                dialog.DbType = dbType;
                bool? result = dialog.ShowModal();
                if (result.HasValue && result.Value == true)
                {
                    if (!string.IsNullOrWhiteSpace(dialog.ConnectionString))
                    {
                        Helpers.DataConnectionHelper.SaveDataConnection(dialog.ConnectionString, dbType);
                        ExplorerControl control = _parentWindow.Content as ExplorerControl;
                        control.BuildDatabaseTree();
                        Helpers.DataConnectionHelper.LogUsage("DatabasesAddCeDatabase");
                    }
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, dbType);
            }
        }

        public void AddCe35Database(object sender, ExecutedRoutedEventArgs e)
        {
            // http://www.mztools.com/articles/2007/MZ2007011.aspx
            if (!Helpers.DataConnectionHelper.ProviderIsInstalled(new Guid(Resources.SqlCompact35Provider)))
            {
                MessageBox.Show("The version 3.5 Visual Studio DDEX provider is not installed, cannot add connection");
                return;
            }
            try
            {
                var objIVsDataConnectionDialogFactory = package.GetServiceHelper(typeof(IVsDataConnectionDialogFactory)) as IVsDataConnectionDialogFactory;
                var objIVsDataConnectionDialog = objIVsDataConnectionDialogFactory.CreateConnectionDialog();
                objIVsDataConnectionDialog.AddAllSources();
                objIVsDataConnectionDialog.SelectedSource = new Guid("130BADA6-E128-423c-9D07-02E4734D45D4");
                objIVsDataConnectionDialog.SelectedProvider = new Guid(Resources.SqlCompact35Provider);

                if (objIVsDataConnectionDialog.ShowDialog() && objIVsDataConnectionDialog.SelectedProvider == new Guid(Resources.SqlCompact35Provider))
                {
                    Helpers.DataConnectionHelper.SaveDataConnection(package, objIVsDataConnectionDialog.EncryptedConnectionString, objIVsDataConnectionDialog.DisplayConnectionString, Resources.SqlCompact35InvariantName, new Guid(Resources.SqlCompact35Provider));
                    ExplorerControl control = _parentWindow.Content as ExplorerControl;
                    control.BuildDatabaseTree();
                    Helpers.DataConnectionHelper.LogUsage("DatabasesAddCe35Database");
                }
                objIVsDataConnectionDialog.Dispose();
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE35);
            }
        }

        public void AddCe40Database(object sender, ExecutedRoutedEventArgs e)
        {
            // http://www.mztools.com/articles/2007/MZ2007011.aspx
            if (!Helpers.DataConnectionHelper.ProviderIsInstalled(new Guid(Resources.SqlCompact40Provider)))
            {
                MessageBox.Show("The version 4.0 Visual Studio DDEX provider is not installed, cannot add connection");
                return;
            }
            try
            {
                var objIVsDataConnectionDialogFactory = package.GetServiceHelper(typeof(IVsDataConnectionDialogFactory)) as IVsDataConnectionDialogFactory;
                var objIVsDataConnectionDialog = objIVsDataConnectionDialogFactory.CreateConnectionDialog();
                objIVsDataConnectionDialog.AddAllSources();
                objIVsDataConnectionDialog.SelectedSource = new Guid("34A4B3E8-C54D-466F-92EA-E5814B97CA43");
                objIVsDataConnectionDialog.SelectedProvider = new Guid(Resources.SqlCompact40Provider);

                if (objIVsDataConnectionDialog.ShowDialog() && objIVsDataConnectionDialog.SelectedProvider == new Guid(Resources.SqlCompact40Provider))
                {
                    Helpers.DataConnectionHelper.SaveDataConnection(package, objIVsDataConnectionDialog.EncryptedConnectionString, objIVsDataConnectionDialog.DisplayConnectionString, Resources.SqlCompact40InvariantName, new Guid(Resources.SqlCompact40Provider));
                    ExplorerControl control = _parentWindow.Content as ExplorerControl;
                    control.BuildDatabaseTree();
                    Helpers.DataConnectionHelper.LogUsage("DatabasesAddCe40Database");
                }
                objIVsDataConnectionDialog.Dispose();
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE40, false);
            }
        }

        public void FixConnections(object sender, ExecutedRoutedEventArgs e)
        {
            try
            {
                new DataConnectionHelper().ValidateConnections(package);
                ExplorerControl control = _parentWindow.Content as ExplorerControl;
                control.BuildDatabaseTree();
                Helpers.DataConnectionHelper.LogUsage("DatabasesFixConnections");
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE35, false);
            }
        }

        public void ScriptServerDatabase(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            Scope scope = (Scope)menuItem.Tag;

            var databaseInfo = menuItem.CommandParameter as DatabasesMenuCommandParameters;
            if (databaseInfo == null) return;
            var treeViewItem = databaseInfo.DatabasesTreeViewItem;

            try
            {
                string connectionString = DataConnectionHelper.PromptForConnectionString("Data Source=.");
                if (!string.IsNullOrEmpty(connectionString))
                {                  
                    string fileName;
                    PickTablesDialog ptd = new PickTablesDialog();
                    int totalCount = 0;
                    using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(new DatabaseInfo { ConnectionString = connectionString, DatabaseType = DatabaseType.SQLServer }))
                    {
                        ptd.Tables = repository.GetAllTableNamesForExclusion();
                        totalCount = ptd.Tables.Count;
                    }

                    bool? res = ptd.ShowModal();
                    if (res.HasValue && res.Value == true && (ptd.Tables.Count < totalCount))
                    {
                        SaveFileDialog fd = new SaveFileDialog();
                        fd.Title = "Save generated database script as";
                        fd.Filter = "SQL Server Compact Script (*.sqlce)|*.sqlce|SQL Server Script (*.sql)|*.sql|All Files(*.*)|*.*";
                        fd.OverwritePrompt = true;
                        fd.ValidateNames = true;
                        bool? result = fd.ShowDialog();
                        if (result.HasValue && result.Value == true)
                        {
                            fileName = fd.FileName;
                            using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(new DatabaseInfo { ConnectionString = connectionString, DatabaseType = DatabaseType.SQLServer }))
                            {
                                try
                                {
                                    var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, fd.FileName, DatabaseType.SQLServer);
                                    generator.ExcludeTables(ptd.Tables);
                                    System.Windows.Forms.MessageBox.Show(generator.ScriptDatabaseToFile(scope));
                                    Helpers.DataConnectionHelper.LogUsage("DatabasesScriptServer");
                                }
                                catch (Exception ex)
                                {
                                    Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLServer, false);
                                }
                            }
                        }
                    }
                }
                //objIVsDataConnectionDialog.Dispose();
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLServer, false);
            }
        }

        public void ExportServerDatabaseTo40(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var databaseInfo = menuItem.CommandParameter as DatabasesMenuCommandParameters;
            if (databaseInfo == null) return;
            var treeViewItem = databaseInfo.DatabasesTreeViewItem;

            try
            {
                string connectionString = DataConnectionHelper.PromptForConnectionString("Data Source=.");
                if (!string.IsNullOrEmpty(connectionString))
                {
                    PickTablesDialog ptd = new PickTablesDialog();
                    int totalCount = 0;
                    using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(new DatabaseInfo { ConnectionString = connectionString, DatabaseType = DatabaseType.SQLServer }))
                    {
                        ptd.Tables = repository.GetAllTableNamesForExclusion();
                        totalCount = ptd.Tables.Count;
                    }

                    bool? res = ptd.ShowModal();
                    if (res.HasValue && res.Value == true && (ptd.Tables.Count < totalCount))
                    {
                        string sdfName;
                        SaveFileDialog fd = new SaveFileDialog();
                        fd.Title = "Export as";
                        fd.Filter = "SQL Server Compact 4.0 Database (*.sdf)|*.sdf|All Files(*.*)|*.*";
                        fd.OverwritePrompt = true;
                        fd.ValidateNames = true;
                        bool? result = fd.ShowDialog();
                        if (result.HasValue && result.Value == true)
                        {
                            sdfName = fd.FileName;
                            using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(new DatabaseInfo { ConnectionString = connectionString, DatabaseType = DatabaseType.SQLServer }))
                            {
                                try
                                {
                                    string scriptRoot = System.IO.Path.GetTempFileName();
                                    string tempScript = scriptRoot + ".sqlce";
                                    var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, tempScript, DatabaseType.SQLServer);
                                    generator.ExcludeTables(ptd.Tables);
                                    SetStatus("Scripting server database...");
                                    generator.ScriptDatabaseToFile(Scope.SchemaData);

                                    SetStatus("Creating SQL Server Compact database...");
                                    SqlCeScripting.SqlCeHelper4 helper = new SqlCeScripting.SqlCeHelper4();                                    
                                    string sdfConnectionString = string.Format("Data Source={0};Max Database Size=4091", sdfName);
                                    if (System.IO.File.Exists(sdfName))
                                        File.Delete(sdfName);
                                    helper.CreateDatabase(sdfConnectionString);

                                    BackgroundWorker bw = new BackgroundWorker();
                                    List<string> parameters = new List<string>();
                                    parameters.Add(sdfConnectionString);
                                    parameters.Add(tempScript);
                                    parameters.Add(scriptRoot);

                                    bw.DoWork += new DoWorkEventHandler(bw_DoWork);
                                    bw.RunWorkerCompleted += (s, ea) =>
                                        {
                                            try
                                            {
                                                if (ea.Error != null)
                                                {
                                                    Helpers.DataConnectionHelper.SendError(ea.Error, DatabaseType.SQLCE40, false);
                                                }
                                                Helpers.DataConnectionHelper.LogUsage("DatabasesExportFromServer");
                                            }
                                            finally
                                            {
                                                bw.Dispose();
                                            }
                                        };
                                    bw.RunWorkerAsync(parameters);

                                }
                                catch (Exception ex)
                                {
                                    Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE40, false);
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLServer);
            }
        }

        private void SetStatus(string message)
        {
            int frozen;
            IVsStatusbar statusBar = package.GetServiceHelper(typeof(SVsStatusbar)) as IVsStatusbar;
            statusBar.IsFrozen(out frozen);
            if (!Convert.ToBoolean(frozen))
            {
                statusBar.SetText(message);
            }
        }

        void bw_DoWork(object sender, DoWorkEventArgs e)
        {
            SetStatus("Importing data...");

            var parameters = e.Argument as List<string>;
            string sdfConnectionString = parameters[0];
            string tempScript = parameters[1];
            string scriptRoot = parameters[2];

            using (IRepository ce4Repository = Helpers.DataConnectionHelper.CreateRepository(new DatabaseInfo { ConnectionString = sdfConnectionString, DatabaseType = DatabaseType.SQLCE40 }))
            {
                //Handles large exports also... 
                if (File.Exists(tempScript)) // Single file
                {
                    ce4Repository.ExecuteSqlFile(tempScript);
                }
                else // possibly multiple files - tmp2BB9.tmp_0.sqlce
                {
                    for (int i = 0; i < 400; i++)
                    {
                        string testFile = string.Format("{0}_{1}{2}", scriptRoot, i, ".sqlce");
                        if (File.Exists(testFile))
                        {
                            ce4Repository.ExecuteSqlFile(testFile);
                        }
                    }
                }
            }
            SetStatus("Database file saved");
        }

        public void GenerateServerDgmlFiles(object sender, ExecutedRoutedEventArgs e)
        {
            // http://www.mztools.com/articles/2007/MZ2007011.aspx
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var databaseInfo = menuItem.CommandParameter as DatabasesMenuCommandParameters;
            if (databaseInfo == null) return;

            var treeViewItem = databaseInfo.DatabasesTreeViewItem;
            bool originalValue = Properties.Settings.Default.KeepSchemaNames;
            var dte = package.GetServiceHelper(typeof(EnvDTE.DTE)) as EnvDTE.DTE;
            try
            {
                string connectionString = DataConnectionHelper.PromptForConnectionString("Data Source=.");
                if (!string.IsNullOrEmpty(connectionString))
                {
                    string fileName;
                    PickTablesDialog ptd = new PickTablesDialog();
                    int totalCount = 0;
                    using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(new DatabaseInfo { ConnectionString = connectionString, DatabaseType = DatabaseType.SQLServer }))
                    {
                        ptd.Tables = repository.GetAllTableNamesForExclusion();
                        totalCount = ptd.Tables.Count;
                    }

                    bool? res = ptd.ShowModal();

                    SaveFileDialog fd = new SaveFileDialog();
                    fd.Title = "Save generated DGML file as";
                    fd.Filter = "DGML (*.dgml)|*.dgml";
                    fd.OverwritePrompt = true;
                    fd.ValidateNames = true;
                    bool? result = fd.ShowDialog();
                    if (result.HasValue && result.Value == true)
                    {
                        Properties.Settings.Default.KeepSchemaNames = true;
                        fileName = fd.FileName;
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(new DatabaseInfo { ConnectionString = connectionString, DatabaseType = DatabaseType.SQLServer }))
                        {
                            try
                            {
                                var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, fileName, DatabaseType.SQLServer);
                                generator.GenerateSchemaGraph(connectionString, ptd.Tables);
                                dte.ItemOperations.OpenFile(fileName);
                                dte.ActiveDocument.Activate();
                                Helpers.DataConnectionHelper.LogUsage("DatabasesScriptDGML");
                            }
                            catch (Exception ex)
                            {
                                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLServer, false);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLServer);
            }
            finally
            {
                Properties.Settings.Default.KeepSchemaNames = originalValue;
            }
        }

        public void SyncFxGenerateLocalDatabaseCacheCode(object sender, ExecutedRoutedEventArgs e)
        {
            // http://www.mztools.com/articles/2007/MZ2007011.aspx
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            Scope scope = (Scope)menuItem.Tag;

            var dte = package.GetServiceHelper(typeof(EnvDTE.DTE)) as EnvDTE.DTE;

            var dteH = new Helpers.EnvDTEHelper();

            var project = dteH.GetProject(dte);
            if (project == null)
            {
                MessageBox.Show("Please select a project in Solution Explorer, where you want the SyncFx classes to be placed");
                return;
            }
            if (!dteH.AllowedProjectKinds.Contains(new Guid(project.Kind)))
            {
                MessageBox.Show("The selected project type does not support Sync Framework (please let me know if I am wrong)");
                return;
            }
            if (project.CodeModel.Language != CodeModelLanguageConstants.vsCMLanguageCSharp)
            {
                MessageBox.Show("Unsupported code language, only C# is currently supported");
                return;
            }
            if (project.Properties.Item("TargetFrameworkMoniker") == null)
            {
                MessageBox.Show("The selected project type does not support Sync Framework - missing TargetFrameworkMoniker");
                return;
            }
            if (!project.Properties.Item("TargetFrameworkMoniker").Value.ToString().Contains(".NETFramework"))
            {
                MessageBox.Show("The selected project type does not support .NET Desktop - wrong TargetFrameworkMoniker: " + project.Properties.Item("TargetFrameworkMoniker").Value.ToString());
                return;
            }

            try
            {
                string serverConnectionString = DataConnectionHelper.PromptForConnectionString("Data Source=.");
                if (!string.IsNullOrEmpty(serverConnectionString))
                {         
                    string clientConnectionString = string.Empty;

                    //grab target SQL CE Database
                    var dialog = new ConnectionDialog();
                    dialog.DbType = DatabaseType.SQLCE35;
                    bool? result = dialog.ShowModal();
                    if (result.HasValue && result.Value == true && !string.IsNullOrWhiteSpace(dialog.ConnectionString))
                    {
                        clientConnectionString = dialog.ConnectionString;
                    }
                    else
                    {
                        return;
                    }

                    string model = string.Empty;
                    SyncFxDialog sfd = new SyncFxDialog();
                    int totalCount = 0;
                    var databaseInfo = new DatabaseMenuCommandParameters
                    {
                        DatabaseInfo = new DatabaseInfo
                        {
                            ConnectionString = serverConnectionString,
                            DatabaseType = DatabaseType.SQLServer
                        }
                    };
                    totalCount = SyncFxGetObjectsForSync(sfd, databaseInfo);
                    sfd.ModelName = model;

                    bool? res = sfd.ShowModal();
                    if (res.HasValue && res.Value == true && (sfd.Tables.Count > 0) && !string.IsNullOrWhiteSpace(sfd.ModelName))
                    {
                        model = sfd.ModelName;
                        var defaultNamespace = project.Properties.Item("DefaultNamespace").Value.ToString();

                        var classes = new Dictionary<string, string>();

                        var columns = sfd.Columns.Where(c => sfd.Tables.Contains(c.TableName)).ToList();
                        if (columns == null)
                            return;

                        classes = new SyncFxHelper().GenerateCodeForScope(serverConnectionString, clientConnectionString, "SQL", model, columns, defaultNamespace);
                        var projectPath = project.Properties.Item("FullPath").Value.ToString();

                        foreach (var item in classes)
                        {
                            string fileName = Path.Combine(projectPath, item.Key + ".cs");
                            if (File.Exists(fileName))
                            {
                                File.Delete(fileName);
                            }
                            File.WriteAllText(fileName, item.Value);
                            project.ProjectItems.AddFromFile(fileName);
                        }
                        //Adding references - http://blogs.msdn.com/b/murat/archive/2008/07/30/envdte-adding-a-refernce-to-a-project.aspx
                        EnvDTEHelper.AddReference(project, "System.Data.SqlServerCe, Version=3.5.1.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");

                        EnvDTEHelper.AddReference(project, "Microsoft.Synchronization, Version=2.1.0.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");
                        EnvDTEHelper.AddReference(project, "Microsoft.Synchronization.Data, Version=3.1.0.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");
                        EnvDTEHelper.AddReference(project, "Microsoft.Synchronization.Data.SqlServer, Version=3.1.0.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");
                        EnvDTEHelper.AddReference(project, "Microsoft.Synchronization.Data.SqlServerCe, Version=3.1.0.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");
                        MessageBox.Show("Scope: " + model + " code generated.");
                        Helpers.DataConnectionHelper.LogUsage("DatabasesSyncAddLocalDBCache");
                    }


                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLServer);
            }
        }

        #endregion

        #region Database Level Commands
        public void RemoveCeDatabase(object sender, ExecutedRoutedEventArgs e)
        {
            try
            {
                var databaseInfo = ValidateMenuInfo(sender);
                if (databaseInfo == null) return;

                if (databaseInfo.DatabaseInfo.FromServerExplorer)
                {
                    Guid provider = new Guid(Resources.SqlCompact40Provider);
                    if (databaseInfo.DatabaseInfo.DatabaseType == DatabaseType.SQLCE35)
                    {
                        provider = new Guid(Resources.SqlCompact35Provider);
                    }
                    if (!Helpers.DataConnectionHelper.ProviderIsInstalled(provider))
                    {
                        MessageBox.Show("The DDEX provider is not installed, cannot remove connection");
                        return;
                    }            
                    Helpers.DataConnectionHelper.RemoveDataConnection(package, databaseInfo.DatabaseInfo.ConnectionString, provider);
                }
                else
                {
                    Helpers.DataConnectionHelper.RemoveDataConnection(databaseInfo.DatabaseInfo.ConnectionString);
                }
                ExplorerControl control = _parentWindow.Content as ExplorerControl;
                control.BuildDatabaseTree();
                Helpers.DataConnectionHelper.LogUsage("DatabaseRemoveCe");
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE35);
            }
        }


#region Maintenance menu items
 
        public void ShrinkDatabase(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;
            try
            {
                ISqlCeHelper helper = Helpers.DataConnectionHelper.CreateSqlCeHelper(databaseInfo.DatabaseInfo.DatabaseType);
                helper.ShrinkDatabase(databaseInfo.DatabaseInfo.ConnectionString);
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }

        public void CompactDatabase(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;
            try
            {
                ISqlCeHelper helper = Helpers.DataConnectionHelper.CreateSqlCeHelper(databaseInfo.DatabaseInfo.DatabaseType);
                helper.CompactDatabase(databaseInfo.DatabaseInfo.ConnectionString);
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }

        public void VerifyDatabase(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;
            try
            {
                ISqlCeHelper helper = Helpers.DataConnectionHelper.CreateSqlCeHelper(databaseInfo.DatabaseInfo.DatabaseType);
                helper.VerifyDatabase(databaseInfo.DatabaseInfo.ConnectionString);
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }

        public void RepairDatabaseDeleteCorruptedRows(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;
            try
            {
                ISqlCeHelper helper = Helpers.DataConnectionHelper.CreateSqlCeHelper(databaseInfo.DatabaseInfo.DatabaseType);
                helper.RepairDatabaseDeleteCorruptedRows(databaseInfo.DatabaseInfo.ConnectionString);
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }

        public void RepairDatabaseRecoverAllOrFail(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;
            try
            {
                ISqlCeHelper helper = Helpers.DataConnectionHelper.CreateSqlCeHelper(databaseInfo.DatabaseInfo.DatabaseType);
                helper.RepairDatabaseRecoverAllOrFail(databaseInfo.DatabaseInfo.ConnectionString);
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }

        public void RepairDatabaseRecoverAllPossibleRows(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;
            try
            {
                ISqlCeHelper helper = Helpers.DataConnectionHelper.CreateSqlCeHelper(databaseInfo.DatabaseInfo.DatabaseType);
                helper.RepairDatabaseRecoverAllPossibleRows(databaseInfo.DatabaseInfo.ConnectionString);
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }

        public void BuildTable(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;
            try
            {
                using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                {
                    var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, databaseInfo.DatabaseInfo.DatabaseType);
                    TableBuilderDialog tbd = new TableBuilderDialog(null);
                    if (tbd.ShowModal() == true)
                    {
                        generator.GenerateTableCreate(tbd.TableName, tbd.TableColumns);
                        var script = generator.GeneratedScript.ToString();
                        if (!string.IsNullOrEmpty(tbd.PkScript))
                        {
                            script += tbd.PkScript;
                        }
                        SpawnSqlEditorWindow(databaseInfo.DatabaseInfo, script);
                        Helpers.DataConnectionHelper.LogUsage("TableBuild");
                    }
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType);
            }
        }

#endregion

        private void SpawnSqlEditorWindow(DatabaseInfo databaseInfo,  string sqlScript)
        {
            try
            {
                if (databaseInfo == null) return;
                if (package == null) return;
                Debug.Assert(package != null, "Package property of the Explorere Tool Window should never be null, have you tried to create it manually and not through FindToolWindow()?");

                var sqlEditorWindow = package.CreateWindow<SqlEditorWindow>(Math.Abs(databaseInfo.ConnectionString.GetHashCode()));
                var editorControl = sqlEditorWindow.Content as SqlEditorControl;
                Debug.Assert(editorControl != null);
                editorControl.DatabaseInfo = databaseInfo;
                editorControl.ExplorerControl = _parentWindow.Content as ExplorerControl;
                editorControl.SqlText = sqlScript;
                sqlEditorWindow.Caption = databaseInfo.Caption;
                Helpers.DataConnectionHelper.LogUsage("DatabaseOpenEditor");
                Debug.Assert(editorControl != null, "The SqlEditorWindow *should* have a editorControl with content.");
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLServer);
            }

        }

        public void SpawnSqlEditorWindow(object sender, ExecutedRoutedEventArgs e)
        {
            try
            {
                var databaseInfo = ValidateMenuInfo(sender);
                if (databaseInfo == null) return;
                if (package == null) return;
                Debug.Assert(package != null, "Package property of the Explorere Tool Window should never be null, have you tried to create it manually and not through FindToolWindow()?");

                var sqlEditorWindow = package.CreateWindow<SqlEditorWindow>(Math.Abs(databaseInfo.DatabaseInfo.ConnectionString.GetHashCode()));
                var editorControl = sqlEditorWindow.Content as SqlEditorControl;
                Debug.Assert(editorControl != null);
                editorControl.DatabaseInfo = databaseInfo.DatabaseInfo;
                editorControl.ExplorerControl = _parentWindow.Content as ExplorerControl;
                editorControl.SqlText = "";
                sqlEditorWindow.Caption = databaseInfo.DatabaseInfo.Caption;
                Helpers.DataConnectionHelper.LogUsage("DatabaseOpenEditor");
                Debug.Assert(editorControl != null, "The SqlEditorWindow *should* have a editorControl with content.");
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLServer);
            }
        }

        public void AddDescription(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;
            try
            {
                using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                {
                    string desc = null;
                    ExplorerControl.DescriptionCache = new Helpers.DescriptionHelper().GetDescriptions(databaseInfo.DatabaseInfo);
                    desc = ExplorerControl.DescriptionCache.Where(d => d.Object == null && d.Parent == null).Select(d => d.Description).SingleOrDefault();
                    DescriptionDialog ro = new DescriptionDialog(desc);
                    ro.IsDatabase = true;
                    ro.ShowModal();
                    if (ro.DialogResult.HasValue && ro.DialogResult.Value == true && !string.IsNullOrWhiteSpace(ro.TableDescription) && ro.TableDescription != desc)
                    {
                        new Helpers.DescriptionHelper().SaveDescription(databaseInfo.DatabaseInfo, ExplorerControl.DescriptionCache, ro.TableDescription, null, null);
                    }
                    Helpers.DataConnectionHelper.LogUsage("DatabaseAddDescription");
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType);
            }
        }

        public void ScriptDatabase(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;
            Scope scope = (Scope)menuItem.Tag;

            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;

            try
            {
                int totalCount = 0;
                PickTablesDialog ptd = new PickTablesDialog();
                using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                {
                    ptd.Tables = repository.GetAllTableNamesForExclusion();
                    totalCount = ptd.Tables.Count;
                }

                bool? res = ptd.ShowModal();
                if (res.HasValue && res.Value == true && (ptd.Tables.Count < totalCount))
                {
                    SaveFileDialog fd = new SaveFileDialog();
                    fd.Title = "Save generated database script as";
                    fd.Filter = "SQL Server Compact Script (*.sqlce)|*.sqlce|SQL Server Script (*.sql)|*.sql|All Files(*.*)|*.*";
                    fd.OverwritePrompt = true;
                    fd.ValidateNames = true;
                    bool? result = fd.ShowDialog();
                    if (result.HasValue && result.Value == true)
                    {
                        var fileName = fd.FileName;
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                        {
                            var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, fd.FileName, databaseInfo.DatabaseInfo.DatabaseType);
                            generator.ExcludeTables(ptd.Tables);
                            System.Windows.Forms.MessageBox.Show(generator.ScriptDatabaseToFile(scope));
                            Helpers.DataConnectionHelper.LogUsage("DatabaseScriptCe");
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }

        }

        public void GenerateDiffScript(object sender, ExecutedRoutedEventArgs e)
        {
            try
            {
                var databaseInfo = ValidateMenuInfo(sender);
                if (databaseInfo == null) return;

                Dictionary<string, DatabaseInfo> databaseList = DataConnectionHelper.GetDataConnections(package, true, false);
                foreach (KeyValuePair<string, DatabaseInfo> info in DataConnectionHelper.GetOwnDataConnections())
                {
                    if (!databaseList.ContainsKey(info.Key))
                        databaseList.Add(info.Key, info.Value);
                }
                foreach (KeyValuePair<string, DatabaseInfo> info in databaseList)
                {
                    string sourceType = string.Empty;
                    switch (info.Value.DatabaseType)
                    {
                        case DatabaseType.SQLCE35:
                            sourceType = "3.5";
                            break;
                        case DatabaseType.SQLCE40:
                            sourceType = "4.0";
                            break;
                        case DatabaseType.SQLServer:
                            sourceType = "Server";
                            break;
                    }
                    info.Value.Caption = string.Format("{0} ({1})", info.Value.Caption, sourceType);
                }

                CompareDialog cd = new CompareDialog(databaseInfo.DatabaseInfo.Caption, databaseList);

                bool? result = cd.ShowModal();
                if (result.HasValue && result.Value == true && (cd.TargetDatabase.Key != null))
                {
                    var target = cd.TargetDatabase;
                    var swap = cd.SwapTarget;

                    var source = new KeyValuePair<string, DatabaseInfo>(databaseInfo.DatabaseInfo.ConnectionString, databaseInfo.DatabaseInfo);
                    if (swap)
                    {
                        source = target;
                        target = new KeyValuePair<string, DatabaseInfo>(databaseInfo.DatabaseInfo.ConnectionString, databaseInfo.DatabaseInfo);
                    }

                    var editorTarget = target;
                    if (editorTarget.Value.DatabaseType == DatabaseType.SQLServer)
                    {
                        editorTarget = source;
                    }

                    using (IRepository sourceRepository = Helpers.DataConnectionHelper.CreateRepository(source.Value))
                    {
                        var generator = Helpers.DataConnectionHelper.CreateGenerator(sourceRepository, databaseInfo.DatabaseInfo.DatabaseType);
                        using (IRepository targetRepository = Helpers.DataConnectionHelper.CreateRepository(target.Value))
                        {
                            try
                            {
                                SqlCeDiff.CreateDiffScript(sourceRepository, targetRepository, generator, Properties.Settings.Default.DropTargetTables);

                                var sqlEditorWindow = package.CreateWindow<SqlEditorWindow>(Math.Abs(editorTarget.Key.GetHashCode()));
                                var editorControl = sqlEditorWindow.Content as SqlEditorControl;
                                editorControl.ExplorerControl = _parentWindow.Content as ExplorerControl;
                                Debug.Assert(editorControl != null);
                                editorControl.DatabaseInfo = editorTarget.Value;

                                string explain = @"-- This database diff script contains the following objects:
-- - Tables:  Any that are not in the destination
-- -          (tables that are only in the destination are NOT dropped, unless that option is set)
-- - Columns: Any added, deleted, changed columns for existing tables
-- - Indexes: Any added, deleted indexes for existing tables
-- - Foreign keys: Any added, deleted foreign keys for existing tables
-- ** Make sure to test against a production version of the destination database! ** " + Environment.NewLine + Environment.NewLine;

                                if (swap)
                                {
                                    explain += "-- ** Please note that the soruce and target have been swapped! ** " + Environment.NewLine + Environment.NewLine;
                                }

                                editorControl.SqlText = explain + generator.GeneratedScript;
                                sqlEditorWindow.Caption = editorTarget.Value.Caption;
                                Helpers.DataConnectionHelper.LogUsage("DatabaseScriptDiff");
                            }
                            catch (Exception ex)
                            {
                                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE35);
                            }

                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE35);
            }

        }

        public void ExportToServer(object sender, ExecutedRoutedEventArgs e)
        {
            try
            {
                var databaseInfo = ValidateMenuInfo(sender);
                if (databaseInfo == null) return;

                Dictionary<string, DatabaseInfo> databaseList = DataConnectionHelper.GetDataConnections(package, true, true);

                ExportDialog cd = new ExportDialog(databaseInfo.DatabaseInfo.Caption, databaseList);

                bool? result = cd.ShowModal();
                if (result.HasValue && result.Value == true && (cd.TargetDatabase.Key != null))
                {
                    using (IRepository ceRepository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                    {
                        string fileName = Path.GetTempFileName();
                        var generator = Helpers.DataConnectionHelper.CreateGenerator(ceRepository, fileName, databaseInfo.DatabaseInfo.DatabaseType);
                        generator.ScriptDatabaseToFile(Scope.SchemaData);
                        using (IRepository serverRepository = Helpers.DataConnectionHelper.CreateRepository(cd.TargetDatabase.Value))
                        {
                            serverRepository.ExecuteSqlFile(fileName);
                            MessageBox.Show("Database exported");
                            Helpers.DataConnectionHelper.LogUsage("DatabaseScriptToServer");
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE35);
            }

        }

        public void UpgradeTo40(object sender, ExecutedRoutedEventArgs e)
        {
            if (MessageBox.Show("This will upgrade the 3.5 database to 4.0 format, and leave a renamed backup of the 3.5 database. Do you wish to proceed?", "SQL Server Compact Toolbox", MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.No) == MessageBoxResult.No)
                return;

            if (!Helpers.DataConnectionHelper.ProviderIsInstalled(new Guid(Resources.SqlCompact40Provider)))
            {
                MessageBox.Show("The version 4.0 Visual Studio DDEX provider is not installed, cannot upgrade");
                return;
            }

            try
            {
                var databaseInfo = ValidateMenuInfo(sender);
                if (databaseInfo == null) return;

                SqlCeScripting.SqlCeHelper4 helper = new SqlCeScripting.SqlCeHelper4();
                    
                string path = helper.PathFromConnectionString(databaseInfo.DatabaseInfo.ConnectionString);

                if (!File.Exists(path))
                {
                    MessageBox.Show(string.Format("Database file in path: {0} could not be found", path));
                    return;
                }

                string newFile = Path.Combine(Path.GetDirectoryName(path), Path.GetFileNameWithoutExtension(path) + "_35" + Path.GetExtension(path));
                if (System.IO.File.Exists(newFile))
                {
                    for (int i = 0; i < 100; i++)
                    {
                        newFile = Path.Combine(Path.GetDirectoryName(newFile),  Path.GetFileNameWithoutExtension(newFile) + "_" + i.ToString() + "." + Path.GetExtension(newFile));
                        if (!File.Exists(newFile))
                            break;
                    }
                }

                if (System.IO.File.Exists(newFile))
                {
                    MessageBox.Show("Could not create unique file name...");
                    return;
                }
                System.IO.File.Copy(path, newFile);
                helper.UpgradeTo40(databaseInfo.DatabaseInfo.ConnectionString);
                MessageBox.Show(string.Format("Database upgraded, version 3.5 database backed up to: {0}", newFile));
                Helpers.DataConnectionHelper.RemoveDataConnection(package, databaseInfo.DatabaseInfo.ConnectionString, new Guid(Resources.SqlCompact35Provider));
                Helpers.DataConnectionHelper.SaveDataConnection(databaseInfo.DatabaseInfo.ConnectionString, DatabaseType.SQLCE40);
                ExplorerControl control = _parentWindow.Content as ExplorerControl;
                control.BuildDatabaseTree();
                Helpers.DataConnectionHelper.LogUsage("DatabaseUpgrade40");
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE35);
            }

        }

        public void GenerateCeDgmlFiles(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;

            if (package == null) return;
            var dte = package.GetServiceHelper(typeof(EnvDTE.DTE)) as EnvDTE.DTE;

            SaveFileDialog fd = new SaveFileDialog();
            fd.Title = "Save generated DGML file as";
            fd.Filter = "DGML (*.dgml)|*.dgml";
            fd.OverwritePrompt = true;
            fd.ValidateNames = true;
            bool? result = fd.ShowDialog();
            if (result.HasValue && result.Value == true)
            {
                var fileName = fd.FileName;
                try
                {
                    using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                    {
                        var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, fileName, databaseInfo.DatabaseInfo.DatabaseType);
                        generator.GenerateSchemaGraph(databaseInfo.DatabaseInfo.Caption, Properties.Settings.Default.IncludeSystemTablesInDocumentation, false);
                        dte.ItemOperations.OpenFile(fileName);
                        dte.ActiveDocument.Activate();
                        Helpers.DataConnectionHelper.LogUsage("DatabaseScriptDGML");
                    }
                }
                catch (Exception ex)
                {
                    Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE35);
                }
            }
        }

        public void GenerateDatabaseInfo(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;

            if (package == null) return;

            try
            {
                using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                {
                    var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, databaseInfo.DatabaseInfo.DatabaseType);
                    generator.GenerateDatabaseInfo();
                    SpawnSqlEditorWindow(databaseInfo.DatabaseInfo, generator.GeneratedScript);
                    Helpers.DataConnectionHelper.LogUsage("DatabaseScriptInfo");
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }


        public void GenerateDocFiles(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;

            if (package == null) return;
            var dte = package.GetServiceHelper(typeof(EnvDTE.DTE)) as EnvDTE.DTE;

            SaveFileDialog fd = new SaveFileDialog();
            fd.Title = "Save generated documentation as";
            //fd.Filter = "HTML (*.html)|*.html|WikiPlex (*.wiki)|*.wiki|Raw XML (*.xml)|*.xml";
            fd.Filter = "HTML (*.html)|*.html|Raw XML (*.xml)|*.xml";
            fd.OverwritePrompt = true;
            fd.ValidateNames = true;
            bool? result = fd.ShowDialog();
            if (result.HasValue && result.Value == true)
            {
                var fileName = fd.FileName;
                try
                {
                    var sqlCeDoc = new SqlCeDbDoc();
                    sqlCeDoc.CreateDocumentation(databaseInfo.DatabaseInfo, fileName, true, null);
                    if (System.IO.File.Exists(fileName))
                    {
                        System.Diagnostics.Process.Start(fileName);
                    }
                    Helpers.DataConnectionHelper.LogUsage("DatabaseScriptDoc");
                }
                catch (Exception ex)
                {
                    Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE35);
                }
            }

        }


        public void GenerateEdmxInProject(object sender, ExecutedRoutedEventArgs e)
        {

            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;

            try
            {
                if (package == null) return;
                var dte = package.GetServiceHelper(typeof(EnvDTE.DTE)) as EnvDTE.DTE;
                if (dte.Mode == vsIDEMode.vsIDEModeDebug)
                {
                    MessageBox.Show("Cannot generate code while debugging");
                    return;
                }

                var dteH = new Helpers.EnvDTEHelper();

                var project = dteH.GetProject(dte);
                if (project == null)
                {
                    MessageBox.Show("Please select a project in Solution Explorer, where you want the EDM to be placed");
                    return;
                }
                if (dte.Solution.SolutionBuild.BuildState == vsBuildState.vsBuildStateNotStarted)
                {
                    MessageBox.Show("Please build the project before proceeding");
                    return;
                }
                if (package.VisualStudioVersion == new Version(12, 0))
                {
                    if (!dteH.ContainsEFSqlCeReference(project))
                    {
                        MessageBox.Show("Please add the EntityFramework.SqlServerCompact NuGet package to the project");
                        return;
                    }
                }
                if (!dteH.AllowedProjectKinds.Contains(new Guid(project.Kind)))
                {
                    MessageBox.Show("The selected project type does not support Entity Framework (please let me know if I am wrong)");
                    return;
                }

                if (project.Properties.Item("TargetFrameworkMoniker") == null)
                {
                    MessageBox.Show("The selected project type does not have a TargetFrameworkMoniker");
                    return;
                }
                if (!project.Properties.Item("TargetFrameworkMoniker").Value.ToString().Contains(".NETFramework"))
                {
                    MessageBox.Show("The selected project type does not support .NET Desktop - wrong TargetFrameworkMoniker: " + project.Properties.Item("TargetFrameworkMoniker").Value.ToString());
                    return;
                }

                string provider = Resources.SqlCompact40InvariantName;

                if (databaseInfo.DatabaseInfo.DatabaseType == DatabaseType.SQLCE35)
                    provider = Resources.SqlCompact35InvariantName;

                string model = Remove(Path.GetFileNameWithoutExtension(databaseInfo.DatabaseInfo.Caption), new List<char> { '#', '.', '+', '-', ' ' });
                EdmxDialog edmxDialog = new EdmxDialog();
                edmxDialog.ModelName = model;
                edmxDialog.ProjectName = project.Name;
                if (package.VisualStudioVersion == new Version(12, 0))
                    edmxDialog.HideAddPrivateConfig();
                
                using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                {
                    var tables = repository.GetAllTableNames();
                    var pks = repository.GetAllPrimaryKeys();
                    var checkedTables = new List<string>();
                    foreach (string tableName in tables)
                    {
                        var pk = pks.Where(k => k.TableName == tableName).FirstOrDefault();
                        if (pk.TableName != null)
                        {
                            checkedTables.Add(tableName);
                        }
                    }
                    edmxDialog.Tables = checkedTables;
                }

                bool? result = edmxDialog.ShowModal();
                if (result.HasValue && result.Value == true && (!string.IsNullOrWhiteSpace(edmxDialog.ModelName)))
                {
                    model = edmxDialog.ModelName;
                    string edmxPath = Path.Combine(Path.GetTempPath(), model + ".edmx");
                    Version ver = new Version(2, 0, 0, 0);
                    if (databaseInfo.DatabaseInfo.DatabaseType == DatabaseType.SQLCE40
                        && package.VisualStudioVersion == new Version(12, 0))
                    {
                        ver = new Version(3, 0, 0, 0);
                    }
                    EdmGen2.EdmGen2.ModelGen(databaseInfo.DatabaseInfo.ConnectionString, provider, model, Path.GetTempPath(), edmxDialog.ForeignKeys, edmxDialog.Pluralize, edmxDialog.Tables, ver);
                    if (EdmGen2.EdmGen2.Errors.Count > 0)
                    {
                        var sb = new System.Text.StringBuilder();
                        foreach (var item in EdmGen2.EdmGen2.Errors)
                        {
                            sb.AppendLine(item);
                        }
                        MessageBox.Show("Errors encountered during edmx generation" + System.Environment.NewLine + sb.ToString());
                    }
                    if (!File.Exists(edmxPath))
                    {
                        return;
                    }

                    bool proceed = true;
                    ProjectItem edmxItem = dteH.GetProjectEdmx(project, model);
                    if (edmxItem == null)
                    {
                        project.ProjectItems.AddFromFileCopy(edmxPath);
                    }
                    else
                    {
                        if (MessageBox.Show("The Entity Data Model already exists in the project, do you wish to replace it?", "EDMX Generator", MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.No) == MessageBoxResult.Yes)
                        {
                            proceed = true;
                            edmxItem.Delete();
                            project.ProjectItems.AddFromFileCopy(edmxPath);

                        }
                        else
                        {
                            proceed = false;
                        }
                    }
                    if (package.VisualStudioVersion == new Version(12, 0))
                    {
                        ProjectItem addedItem = dteH.GetProjectEdmx(project, model);
                        if (addedItem != null)
                        {
                            if (project.CodeModel.Language == CodeModelLanguageConstants.vsCMLanguageCSharp)
                            {
                                var template = File.ReadAllText(Path.Combine(dteH.GetVisualStudioInstallationDir(), "ItemTemplates\\CSharp\\Data\\1033\\DbCtxCSEF6\\CSharpDbContext.Context.tt"));
                                template = template.Replace("$edmxInputFile$", model + ".edmx");
                                File.WriteAllText(Path.Combine(Path.GetTempPath(),  model + ".Context.tt"), template);
                                addedItem.ProjectItems.AddFromFileCopy(Path.Combine(Path.GetTempPath(),  model + ".Context.tt"));

                                template = File.ReadAllText(Path.Combine(dteH.GetVisualStudioInstallationDir(), "ItemTemplates\\CSharp\\Data\\1033\\DbCtxCSEF6\\CSharpDbContext.Types.tt"));
                                template = template.Replace("$edmxInputFile$", model + ".edmx");
                                File.WriteAllText(Path.Combine(Path.GetTempPath(), model + ".tt"), template);
                                addedItem.ProjectItems.AddFromFileCopy(Path.Combine(Path.GetTempPath(),  model + ".tt"));
                            }
                            if (project.CodeModel.Language == CodeModelLanguageConstants.vsCMLanguageVB)
                            {
                                var template = File.ReadAllText(Path.Combine(dteH.GetVisualStudioInstallationDir(), "ItemTemplates\\VisualBasic\\Data\\1033\\DbCtxVBEF6\\VBDbContext.Context.tt"));
                                template = template.Replace("$edmxInputFile$", model + ".edmx");
                                File.WriteAllText(Path.Combine(Path.GetTempPath(), model + ".Context.tt"), template);
                                addedItem.ProjectItems.AddFromFileCopy(Path.Combine(Path.GetTempPath(), model + ".Context.tt"));

                                template = File.ReadAllText(Path.Combine(dteH.GetVisualStudioInstallationDir(), "ItemTemplates\\VisualBasic\\Data\\1033\\DbCtxVBEF6\\CSharpDbContext.Types.tt"));
                                template = template.Replace("$edmxInputFile$", model + ".edmx");
                                File.WriteAllText(Path.Combine(Path.GetTempPath(), model + ".tt"), template);
                                addedItem.ProjectItems.AddFromFileCopy(Path.Combine(Path.GetTempPath(), model + ".tt"));
                            }
                            string diagramPath = Path.Combine(Path.GetTempPath(), model + "edmx.diagram");
                            File.WriteAllText(diagramPath, EdmGen2.EdmGen2.GetEDMXDiagram());
                            addedItem.ProjectItems.AddFromFileCopy(diagramPath);
                        }
                        project.Save();
                    }
                    if (edmxDialog.SaveConfig && proceed)
                    {
                        string prefix = "App";
                        string configPath = Path.Combine(Path.GetTempPath(), prefix + ".config");

                        ProjectItem item = dteH.GetProjectConfig(project);
                        if (item == null)
                        {
                            //Add app.config file to project
                            var cfgSb = new System.Text.StringBuilder();
                            cfgSb.AppendLine("<?xml version=\"1.0\" encoding=\"utf-8\" ?>");
                            cfgSb.AppendLine("<configuration>");
                            cfgSb.AppendLine("</configuration>");
                            File.WriteAllText(configPath, cfgSb.ToString());
                            item = project.ProjectItems.AddFromFileCopy(configPath);
                        }

                        if (item != null)
                        {
                            Helpers.AppConfigHelper.BuildConfig(databaseInfo.DatabaseInfo.ConnectionString, project.FullName, provider, model, prefix, item.Name);
                            if (edmxDialog.AddPrivateConfig)
                            {
                                Helpers.AppConfigHelper.WriteSettings(item.FileNames[0], databaseInfo.DatabaseInfo.DatabaseType);
                            }
                        }
                    }
                    Helpers.DataConnectionHelper.LogUsage("DatabaseCreateEDMX");
                }
            }
            // EDM end
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }


        public void GenerateDataContextInProject(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;

            bool isDesktop = false;

            if ((bool)((MenuItem)sender).Tag == true)
            {
                isDesktop = true;
            }

            if (package == null) return;
            var dte = package.GetServiceHelper(typeof(EnvDTE.DTE)) as EnvDTE.DTE;
            if (dte.Mode == vsIDEMode.vsIDEModeDebug)
            {
                MessageBox.Show("Cannot generate code while debugging");
                return;                
            }

            var dteH = new Helpers.EnvDTEHelper();

            var project = dteH.GetProject(dte);
            if (project == null)
            {
                MessageBox.Show("Please select a project in Solution Explorer, where you want the DataContext to be placed");
                return;
            }
            if (!isDesktop && !dteH.AllowedWPProjectKinds.Contains(new Guid(project.Kind)))
            {
                MessageBox.Show("The selected project type does not support Windows Phone (please let me know if I am wrong)");
                return;
            }
            if (isDesktop && !dteH.AllowedProjectKinds.Contains(new Guid(project.Kind)))
            {
                MessageBox.Show("The selected project type does not support LINQ to SQL (please let me know if I am wrong)");
                return;
            }
            if (project.Properties.Item("TargetFrameworkMoniker") == null)
            {
                MessageBox.Show("The selected project type does not support Windows Phone - missing TargetFrameworkMoniker");
                return;
            }
            if (!isDesktop)
            {
                if (project.Properties.Item("TargetFrameworkMoniker").Value.ToString() == "Silverlight,Version=v4.0,Profile=WindowsPhone71" || project.Properties.Item("TargetFrameworkMoniker").Value.ToString() == "WindowsPhone,Version=v8.0")
                { }
                else
                {
                    MessageBox.Show("The selected project type does not support Windows Phone 7.1/8.0 - wrong TargetFrameworkMoniker: " + project.Properties.Item("TargetFrameworkMoniker").Value.ToString());
                    return;
                }
            }
            if (isDesktop && !project.Properties.Item("TargetFrameworkMoniker").Value.ToString().Contains(".NETFramework"))
            {
                MessageBox.Show("The selected project type does not support .NET Desktop - wrong TargetFrameworkMoniker: " + project.Properties.Item("TargetFrameworkMoniker").Value.ToString());
                return;
            }
            if (!isDesktop && databaseInfo.DatabaseInfo.DatabaseType != DatabaseType.SQLCE35)
            {
                MessageBox.Show("Sorry, only version 3.5 databases are supported for now");
                return;
            }

            string sqlMetalPath = (string)Registry.GetValue(@"HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Microsoft SDKs\Windows\v7.0A\WinSDK-NetFx40Tools", "InstallationFolder", null);
            if (string.IsNullOrEmpty(sqlMetalPath))
            {
                sqlMetalPath = (string)Registry.GetValue(@"HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Microsoft SDKs\Windows\v8.0A", "InstallationFolder", string.Empty) + "bin\\NETFX 4.0 Tools\\";
                if (string.IsNullOrEmpty(sqlMetalPath))
                {
                    MessageBox.Show("Could not find SQLMetal location in registry");
                    return;
                }
            } 
            sqlMetalPath = Path.Combine(sqlMetalPath, "sqlmetal.exe");
            if (!File.Exists(sqlMetalPath))
            {
                MessageBox.Show("Could not find SqlMetal in the expected location: " + sqlMetalPath);
                return;
            }
            string sdfFileName = string.Empty;

            try
            {
                using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                {
                    var tables = repository.GetAllTableNames();
                    var pks = repository.GetAllPrimaryKeys();
                    string checkedTables = string.Empty;
                    foreach (string tableName in tables)
                    {
                        var pk = pks.Where(k => k.TableName == tableName).FirstOrDefault();
                        if (pk.TableName == null)
                        {
                            checkedTables += tableName + Environment.NewLine;
                        }
                    }
                    if (!string.IsNullOrEmpty(checkedTables))
                    {
                        string message = string.Format("The tables below do not have Primary Keys defined,{0}and will not be generated properly:{1}{2}", Environment.NewLine, Environment.NewLine, checkedTables);
                        MessageBox.Show(message);
                    }
                    List<KeyValuePair<string, string>> dbInfo = repository.GetDatabaseInfo();
                    foreach (var kvp in dbInfo)
                    {
                        if (kvp.Key == "Database")
                        {
                            sdfFileName = kvp.Value;
                            break;
                        }
                    }
                    sdfFileName = Path.GetFileName(sdfFileName);
                }

                string model = Path.GetFileNameWithoutExtension(databaseInfo.DatabaseInfo.Caption).Replace(" ", string.Empty).Replace("#", string.Empty).Replace(".", string.Empty).Replace("-", string.Empty);
                model = model + "Context";
                DataContextDialog dcDialog = new DataContextDialog();
                dcDialog.ModelName = model;
                dcDialog.IsDesktop = isDesktop;
                dcDialog.ProjectName = project.Name;
                dcDialog.NameSpace = project.Properties.Item("DefaultNamespace").Value.ToString();
                if (EnvDTEHelper.VBProject == new Guid(project.Kind))
                {
                    dcDialog.CodeLanguage = "VB";
                }
                else
                {
                    dcDialog.CodeLanguage = "C#";
                }
                bool? result = dcDialog.ShowModal();
                if (result.HasValue && result.Value == true && (!string.IsNullOrWhiteSpace(dcDialog.ModelName)))
                {
                    if (dcDialog.AddRowversionColumns)
                    {
                        AddRowVersionColumns(databaseInfo);
                    }

                    string sdfPath = databaseInfo.DatabaseInfo.ConnectionString;

                    //If version 4.0, create a 3.5 schema sdf, and use that as connection string
                    if (isDesktop && databaseInfo.DatabaseInfo.DatabaseType == DatabaseType.SQLCE40)
                    {
                        var tempFile = Path.GetTempFileName();
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                        {
                            var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, tempFile, databaseInfo.DatabaseInfo.DatabaseType);
                            generator.ScriptDatabaseToFile(Scope.Schema);
                        }
                        sdfPath = Path.Combine(Path.GetTempPath(), sdfFileName);
                        using (Stream stream = new MemoryStream(Resources.SqlCe35AddinStore))
                        {
                            // Create a FileStream object to write a stream to a file 
                            using (FileStream fileStream = File.Create(sdfPath, (int)stream.Length))
                            {
                                // Fill the bytes[] array with the stream data 
                                byte[] bytesInStream = new byte[stream.Length];
                                stream.Read(bytesInStream, 0, (int)bytesInStream.Length);
                                // Use FileStream object to write to the specified file 
                                fileStream.Write(bytesInStream, 0, bytesInStream.Length);
                            }
                        }
                        DatabaseInfo info = new DatabaseInfo();
                        info.ConnectionString = "Data Source=" + sdfPath;
                        info.DatabaseType = DatabaseType.SQLCE35;
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(info))
                        {
                            string script = File.ReadAllText(tempFile);
                            repository.ExecuteSql(script);
                        }
                        sdfPath = info.ConnectionString;
                    }

                    int versionNumber = GetVersionTableNumber(databaseInfo.DatabaseInfo, isDesktop);

                    model = dcDialog.ModelName;
                    string dcPath = Path.Combine(Path.GetTempPath(), model + ".cs");
                    if (dcDialog.CodeLanguage == "VB")
                    {
                        dcPath = Path.Combine(Path.GetTempPath(), model + ".vb");
                    }
                    string parameters = " /provider:SQLCompact /code:\"" + dcPath + "\"";
                    parameters += " /conn:\"" + sdfPath + "\"";
                    parameters += " /context:" + model;
                    if (dcDialog.Pluralize)
                    {
                        parameters += " /pluralize";
                    }
                    if (!string.IsNullOrWhiteSpace(dcDialog.NameSpace))
                    {
                        parameters += " /namespace:" + dcDialog.NameSpace;
                    }
                    var dcH = new ErikEJ.SqlCeScripting.DataContextHelper();

                    string sqlmetalResult = dcH.RunSqlMetal(sqlMetalPath, parameters);

                    if (!File.Exists(dcPath))
                    {
                        MessageBox.Show("Error during SQL Metal run: " + sqlmetalResult);
                        return;
                    }

                    if (!isDesktop)
                    {
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                        {
                            if (dcDialog.CodeLanguage == "VB")
                            {
                                DataContextHelper.FixDataContextVB(dcPath, model, dcDialog.NameSpace, sdfFileName, repository);
                            }
                            else
                            {
                                DataContextHelper.FixDataContextCS(dcPath, model, dcDialog.NameSpace, sdfFileName, repository);
                            }
                        }
                    }
                    if (dcDialog.MultipleFiles)
                    {
                        Dictionary<string, string> classes = DataContextHelper.SplitIntoMultipleFiles(dcPath, dcDialog.NameSpace, model);
                        string projectPath = project.Properties.Item("FullPath").Value.ToString();

                        foreach (var item in classes)
                        {
                            string fileName = Path.Combine(projectPath, item.Key + ".cs");
                            if (File.Exists(fileName))
                            {
                                File.Delete(fileName);
                            }
                            File.WriteAllText(fileName, item.Value);
                            ProjectItem classItem = dteH.GetProjectDataContextClass(project, fileName);
                            if (classItem != null)
                            {
                                classItem.Delete();
                            }
                            project.ProjectItems.AddFromFile(fileName);
                        }

                    }
                    else
                    {
                        string extension = ".cs";
                        if (dcDialog.CodeLanguage == "VB")
                            extension = ".vb";
                        ProjectItem dcItem = dteH.GetProjectDC(project, model, extension);
                        if (dcItem == null)
                        {
                            project.ProjectItems.AddFromFileCopy(dcPath);
                        }
                        else
                        {
                            if (MessageBox.Show("The Data Context class already exists in the project, do you wish to replace it?", "Data Context Generator", MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.No) == MessageBoxResult.Yes)
                            {
                                dcItem.Delete();
                                project.ProjectItems.AddFromFileCopy(dcPath);
                            }
                        }
                    }
                    EnvDTEHelper.AddReference(project, "System.Data.Linq");
                    if (dcDialog.AddConnectionStringBuilder)
                    {
                        string projectPath = project.Properties.Item("FullPath").Value.ToString();

                        string fileName = "LocalDatabaseConnectionStringBuilder.cs";

                        string filePath = Path.Combine(projectPath, fileName);
                        if (File.Exists(filePath))
                        {
                            File.Delete(filePath);
                        }
                        using (Stream stream = new MemoryStream(Resources.LocalDatabaseConnectionStringBuilder))
                        {
                            // Create a FileStream object to write a stream to a file 
                            using (FileStream fileStream = File.Create(filePath, (int)stream.Length))
                            {
                                // Fill the bytes[] array with the stream data 
                                byte[] bytesInStream = new byte[stream.Length];
                                stream.Read(bytesInStream, 0, (int)bytesInStream.Length);
                                // Use FileStream object to write to the specified file 
                                fileStream.Write(bytesInStream, 0, bytesInStream.Length);
                            }
                        }
                        project.ProjectItems.AddFromFile(filePath);
                    }

                    // Creates __Version table and adds one row if desired
                    if (dcDialog.AddVersionTable)
                    {
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
                        {
                            var list = repository.GetAllTableNames();
                            if (!list.Contains("__VERSION"))
                            {
                                repository.ExecuteSql(string.Format(@"
                                CREATE TABLE [__VERSION] (
                                  [SchemaVersion] int NOT NULL
                                , [DateUpdated] datetime NOT NULL DEFAULT (GETDATE())
                                );
                                GO
                                CREATE INDEX [IX_SchemaVersion] ON [__VERSION] ([SchemaVersion] DESC);
                                GO
                                INSERT INTO [__VERSION] ([SchemaVersion]) VALUES ({0});
                                GO", versionNumber));
                            }
                        }

                    }
                    Helpers.DataConnectionHelper.LogUsage("DatabaseCreateDC");
                }

            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }

        private int GetVersionTableNumber(DatabaseInfo databaseInfo, bool isDesktop)
        {
            if (isDesktop)
                return 0;

            int version = 0;
            using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo))
            {
                var list = repository.GetAllTableNames();
                if (list.Contains("__VERSION"))
                {
                    System.Data.DataSet ds = repository.ExecuteSql(@"
                                SELECT MAX([SchemaVersion]) FROM __VERSION;
                                GO");
                    if (ds != null && ds.Tables.Count > 0)
                    {
                        version = int.Parse(ds.Tables[0].Rows[0][0].ToString());
                    }

                    repository.ExecuteSql(@"
                                DROP TABLE [__VERSION];
                                GO");
                }
            }

            return version;
        }



        public void SyncFxDeprovisionDatabase(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;

            if (databaseInfo.DatabaseInfo.DatabaseType != DatabaseType.SQLCE35)
            {
                MessageBox.Show("Sorry, only version 3.5 databases are supported for now");
                return;
            }
            if (!Helpers.SyncFxHelper.IsProvisioned(databaseInfo.DatabaseInfo))
            {
                MessageBox.Show("The database is not provisioned, cannot deprovision");
                return;                
            }
            try
            {
                bool deprovisioned = new SyncFxHelper().DeprovisionDatabase(databaseInfo.DatabaseInfo.ConnectionString);
                databaseInfo.ExplorerControl.RefreshTables(databaseInfo.DatabaseInfo);
                MessageBox.Show("Database deprovisioned");
                Helpers.DataConnectionHelper.LogUsage("DatabaseSyncDeprovision");
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }

        }

        public void SyncFxGenerateSnapshot(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;

            if (databaseInfo.DatabaseInfo.DatabaseType != DatabaseType.SQLCE35)
            {
                MessageBox.Show("Sorry, only version 3.5 databases are supported for now");
                return;
            }
            if (!Helpers.SyncFxHelper.IsProvisioned(databaseInfo.DatabaseInfo))
            {
                MessageBox.Show("The database is not provisioned, cannot generate snapshots");
                return;
            }

            var fd = new SaveFileDialog();
            fd.Title = "Save generated snapshot database file as";
            fd.Filter = "SQL Compact Database (*.sdf)|*.sdf";
            fd.OverwritePrompt = true;
            fd.ValidateNames = true;
            bool? result = fd.ShowDialog();
            if (result.HasValue && result.Value == true)
            {
                var fileName = fd.FileName;
                try
                {
                    SyncFxHelper.GenerateSnapshot(databaseInfo.DatabaseInfo.ConnectionString, fileName);
                    MessageBox.Show("Database snapshot generated.");
                    Helpers.DataConnectionHelper.LogUsage("DatabaseSyncSnapshot");
                }
                catch (Exception ex)
                {
                    Helpers.DataConnectionHelper.SendError(ex, DatabaseType.SQLCE35, false);
                }
            }
        }

        public void SyncFxProvisionScope(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;

            if (databaseInfo.DatabaseInfo.DatabaseType != DatabaseType.SQLCE35)
            {
                MessageBox.Show("Sorry, only version 3.5 databases are supported for now");
                return;
            }

            try
            {
                string model = Path.GetFileNameWithoutExtension(databaseInfo.DatabaseInfo.Caption).Replace(" ", string.Empty).Replace("#", string.Empty).Replace(".", string.Empty).Replace("-", string.Empty);
                SyncFxDialog sfd = new SyncFxDialog();
                int totalCount = 0;
                totalCount = SyncFxGetObjectsForSync(sfd, databaseInfo);
                sfd.ModelName = model;

                bool? res = sfd.ShowModal();

                if (res.HasValue && res.Value == true && (sfd.Tables.Count > 0))
                {
                    if (SyncFxHelper.SqlCeScopeExists(databaseInfo.DatabaseInfo.ConnectionString, model))
                    {
                        MessageBox.Show("Scope name is already in use. Please enter a different scope name.");
                        return;
                    }

                    model = sfd.ModelName;
                    new SyncFxHelper().ProvisionScope(databaseInfo.DatabaseInfo.ConnectionString, model, sfd.Columns.Where(c => sfd.Tables.Contains(c.TableName)).ToList());
                    MessageBox.Show("Scope: " + model + " has been provisioned.");
                    Helpers.DataConnectionHelper.LogUsage("DatabaseSyncProvision");
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }

        public void SyncFxGenerateSyncCodeInProject(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;

            if (package == null) return;
            var dte = package.GetServiceHelper(typeof(EnvDTE.DTE)) as EnvDTE.DTE;

            var dteH = new Helpers.EnvDTEHelper();

            var project = dteH.GetProject(dte);
            if (project == null)
            {
                MessageBox.Show("Please select a project in Solution Explorer, where you want the SyncFx classes to be placed");
                return;
            }
            if (!dteH.AllowedProjectKinds.Contains(new Guid(project.Kind)))
            {
                MessageBox.Show("The selected project type does not support Sync Framework (please let me know if I am wrong)");
                return;
            }
            if (project.CodeModel.Language != CodeModelLanguageConstants.vsCMLanguageCSharp)
            {
                MessageBox.Show("Unsupported code language, only C# is currently supported");
                return;
            }
            if (project.Properties.Item("TargetFrameworkMoniker") == null)
            {
                MessageBox.Show("The selected project type does not support Sync Framework - missing TargetFrameworkMoniker");
                return;
            }
            if (!project.Properties.Item("TargetFrameworkMoniker").Value.ToString().Contains(".NETFramework"))
            {
                MessageBox.Show("The selected project type does not support .NET Desktop - wrong TargetFrameworkMoniker: " + project.Properties.Item("TargetFrameworkMoniker").Value.ToString());
                return;
            }
            if (databaseInfo.DatabaseInfo.DatabaseType != DatabaseType.SQLCE35)
            {
                MessageBox.Show("Sorry, only version 3.5 databases are supported for now");
                return;
            }

            try
            {
                string model = Path.GetFileNameWithoutExtension(databaseInfo.DatabaseInfo.Caption).Replace(" ", string.Empty).Replace("#", string.Empty).Replace(".", string.Empty).Replace("-", string.Empty);
                SyncFxDialog sfd = new SyncFxDialog();
                int totalCount = 0;
                totalCount = SyncFxGetObjectsForSync(sfd, databaseInfo);
                sfd.ModelName = model;

                bool? res = sfd.ShowModal();
                if (res.HasValue && res.Value == true && (sfd.Tables.Count > 0))
                {
                    model = sfd.ModelName;
                    var defaultNamespace = project.Properties.Item("DefaultNamespace").Value.ToString();

                    var classes = new Dictionary<string, string>();

                    classes = new SyncFxHelper().GenerateCodeForScope(string.Empty, databaseInfo.DatabaseInfo.ConnectionString, "SQLCE", model, sfd.Columns.Where(c => sfd.Tables.Contains(c.TableName)).ToList(), defaultNamespace);
                    var projectPath = project.Properties.Item("FullPath").Value.ToString();

                    foreach (var item in classes)
                    {
                        string fileName = Path.Combine(projectPath, item.Key + ".cs");
                        if (File.Exists(fileName))
                        {
                            File.Delete(fileName);
                        }
                        File.WriteAllText(fileName, item.Value);
                        project.ProjectItems.AddFromFile(fileName);
                    }
                    //Adding references - http://blogs.msdn.com/b/murat/archive/2008/07/30/envdte-adding-a-refernce-to-a-project.aspx
                    EnvDTEHelper.AddReference(project, "System.Data.SqlServerCe, Version=3.5.1.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");

                    EnvDTEHelper.AddReference(project, "Microsoft.Synchronization, Version=2.1.0.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");
                    EnvDTEHelper.AddReference(project, "Microsoft.Synchronization.Data, Version=3.1.0.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");
                    EnvDTEHelper.AddReference(project, "Microsoft.Synchronization.Data.SqlServer, Version=3.1.0.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");
                    EnvDTEHelper.AddReference(project, "Microsoft.Synchronization.Data.SqlServerCe, Version=3.1.0.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");
                    MessageBox.Show("Scope: " + model + " code generated.");
                    Helpers.DataConnectionHelper.LogUsage("DatabaseSyncCodegen");
                }
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }

        private static int SyncFxGetObjectsForSync(SyncFxDialog sfd, DatabaseMenuCommandParameters databaseInfo)
        {
            int totalCount;
            using (var repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
            {
                sfd.Tables =
                    repository.GetAllTableNames().Where(
                        t => !t.EndsWith("scope_info") && !t.EndsWith("scope_config") && !t.EndsWith("schema_info") && !t.EndsWith("_tracking")).ToList();
                sfd.Columns =
                    repository.GetAllColumns().Where(
                        t =>
                        !t.TableName.EndsWith("scope_info") && !t.TableName.EndsWith("scope_config") && !t.TableName.EndsWith("schema_info") &&
                        !t.TableName.EndsWith("_tracking")).ToList();
                sfd.PrimaryKeyColumns =
                    repository.GetAllPrimaryKeys().Where(
                        t =>
                        !t.TableName.EndsWith("scope_info") && !t.TableName.EndsWith("scope_config") && !t.TableName.EndsWith("schema_info") &&
                        !t.TableName.EndsWith("_tracking")).ToList();
                totalCount = sfd.Tables.Count;
            }
            return totalCount;
        }


        private static void AddRowVersionColumns(DatabaseMenuCommandParameters databaseInfo)
        {
            using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(databaseInfo.DatabaseInfo))
            {
                var list = repository.GetAllTableNames();
                var allColumns = repository.GetAllColumns();
                foreach (var table in list)
                {
                    if (!table.StartsWith("__"))
                    {
                        var rowVersionCol = allColumns.Where(c => c.TableName == table && c.DataType == "rowversion").SingleOrDefault();
                        if (rowVersionCol == null)
                        {
                            repository.ExecuteSql(string.Format("ALTER TABLE {0} ADD COLUMN VersionColumn rowversion NOT NULL;{1}GO", table, Environment.NewLine));
                        }
                    }
                }
            }
        }

        public void CheckCeVersion(object sender, ExecutedRoutedEventArgs e)
        {
            var helper = Helpers.DataConnectionHelper.CreateSqlCeHelper(DatabaseType.SQLCE40);
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "SQL Server Compact Database (*.sdf)|*.sdf|All Files(*.*)|*.*";
            ofd.CheckFileExists = true;
            ofd.Multiselect = false;
            ofd.ValidateNames = true;
            if (ofd.ShowDialog() == true)
            {
                try
                {
                    var sdfVersion = helper.DetermineVersion(ofd.FileName);
                    string found = "Unknown";
                    switch (sdfVersion)
                    {
                        case SQLCEVersion.SQLCE20:
                            found = "2.0";
                            break;
                        case SQLCEVersion.SQLCE30:
                            found = "3.0/3.1";
                            break;
                        case SQLCEVersion.SQLCE35:
                            found = "3.5";
                            break;
                        case SQLCEVersion.SQLCE40:
                            found = "4.0";
                            break;
                        default:
                            break;
                    }
                    MessageBox.Show(string.Format("{0} is SQL Server Compact version {1}", Path.GetFileName(ofd.FileName), found), "SQL Server Compact Version Detect");
                    Helpers.DataConnectionHelper.LogUsage("DatabaseVersionDetect");
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        public void RefreshTables(object sender, ExecutedRoutedEventArgs e)
        {
            var databaseInfo = ValidateMenuInfo(sender);
            if (databaseInfo == null) return;
            try
            {
                databaseInfo.ExplorerControl.RefreshTables(databaseInfo.DatabaseInfo);
                Helpers.DataConnectionHelper.LogUsage("DatabaseRefreshTables");
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, databaseInfo.DatabaseInfo.DatabaseType, false);
            }
        }

        #endregion

        private string Remove(string s, IEnumerable<char> chars)
        {
            return new string(s.Where(c => !chars.Contains(c)).ToArray());
        }

        private static DatabaseMenuCommandParameters ValidateMenuInfo(object sender)
        {
            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                return menuItem.CommandParameter as DatabaseMenuCommandParameters;
            }
            else
            {
                return null;
            }
        }

    }
}

