﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ErikEJ.SqlCeToolbox.ToolWindows;
using ErikEJ.SqlCeScripting;
using Microsoft.VisualStudio.Data.Interop;
using Microsoft.Win32;

namespace ErikEJ.SqlCeToolbox.Commands
{
    public class IndexMenuCommandsHandler : CommandHandlerBase
    {
        public IndexMenuCommandsHandler(ExplorerToolWindow parent)
        {
            ParentWindow = parent;
        }

        public void ScriptAsCreate(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var menuInfo = menuItem.CommandParameter as MenuCommandParameters;

                if (menuInfo != null)
                {
                    try
                    {
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(menuInfo.DatabaseInfo))
                        {
                            var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, menuInfo.DatabaseInfo.DatabaseType);
                            generator.GenerateIndexScript(menuInfo.Name, menuInfo.Description);
                            OpenSqlEditorToolWindow(menuInfo, generator.GeneratedScript, false);
                            Helpers.DataConnectionHelper.LogUsage("IndexScriptAsCreate");
                        }
                    }
                    catch (Exception ex)
                    {
                        Helpers.DataConnectionHelper.SendError(ex, menuInfo.DatabaseInfo.DatabaseType, false);
                    }
                }
            }
        }

        public void ScriptAsDrop(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var menuInfo = menuItem.CommandParameter as MenuCommandParameters;

                if (menuInfo != null)
                {
                    try
                    {
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(menuInfo.DatabaseInfo))
                        {
                            var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, menuInfo.DatabaseInfo.DatabaseType);
                            generator.GenerateIndexDrop(menuInfo.Name, menuInfo.Description);
                            OpenSqlEditorToolWindow(menuInfo, generator.GeneratedScript, false);
                            Helpers.DataConnectionHelper.LogUsage("IndexScriptAsDrop");
                        }
                    }
                    catch (Exception ex)
                    {
                        Helpers.DataConnectionHelper.SendError(ex, menuInfo.DatabaseInfo.DatabaseType, false);
                    }
                }
            }
        }

        public void ScriptAsStatistics(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var menuInfo = menuItem.CommandParameter as MenuCommandParameters;

                if (menuInfo != null)
                {
                    try
                    {
                        using (IRepository repository = Helpers.DataConnectionHelper.CreateRepository(menuInfo.DatabaseInfo))
                        {
                            var generator = Helpers.DataConnectionHelper.CreateGenerator(repository, menuInfo.DatabaseInfo.DatabaseType);
                            generator.GenerateIndexStatistics(menuInfo.Name, menuInfo.Description);
                            OpenSqlEditorToolWindow(menuInfo, generator.GeneratedScript, false);
                            Helpers.DataConnectionHelper.LogUsage("ColumnScriptAsStatistics");
                        }
                    }
                    catch (Exception ex)
                    {
                        Helpers.DataConnectionHelper.SendError(ex, menuInfo.DatabaseInfo.DatabaseType, false);
                    }
                }
            }
        }

    }
}