﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ErikEJ.SqlCeToolbox.Helpers;
using ErikEJ.SqlCeToolbox.ToolWindows;
using ErikEJ.SqlCeScripting;
using Microsoft.VisualStudio.Data.Interop;
using Microsoft.Win32;
using Microsoft.VisualStudio.Shell;

namespace ErikEJ.SqlCeToolbox.Commands
{
    public class ScopesMenuCommandsHandler
    {
        private readonly ExplorerToolWindow _parentWindow;

        public ScopesMenuCommandsHandler(ExplorerToolWindow parent)
        {
            _parentWindow = parent;
        }

        public void DropScope(object sender, ExecutedRoutedEventArgs e)
        {
            if (MessageBox.Show("Do you really want to deprovision this scope?", "SQL Server Compact Toolbox", MessageBoxButton.YesNo, MessageBoxImage.Warning, MessageBoxResult.No, MessageBoxOptions.None) == MessageBoxResult.No)
            {
                return;
            }

            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var menuInfo = menuItem.CommandParameter as MenuCommandParameters;

                if (menuInfo != null)
                {
                    try
                    {
                        SyncFxHelper.DeprovisionSqlCeScope(menuInfo.DatabaseInfo.ConnectionString, menuInfo.Name);
                        MessageBox.Show("Scope deprovisioned");
                        if (_parentWindow != null && _parentWindow.Content != null)
                        {
                            ExplorerControl control = _parentWindow.Content as ExplorerControl;
                            control.BuildDatabaseTree();
                        }
                        Helpers.DataConnectionHelper.LogUsage("SyncScopeDrop");
                    }
                    catch (Exception ex)
                    {
                        Helpers.DataConnectionHelper.SendError(ex, menuInfo.DatabaseInfo.DatabaseType);
                    }
                }
            }
        }
        
    }
}
