﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ErikEJ.SqlCeToolbox.ToolWindows;
using ErikEJ.SqlCeScripting;
using Microsoft.VisualStudio.Data.Interop;
using Microsoft.Win32;
using Microsoft.VisualStudio.Shell;

namespace ErikEJ.SqlCeToolbox.Commands
{
    public class SubscriptionsMenuCommandsHandler
    {
        private readonly ExplorerToolWindow _parentWindow;

        public SubscriptionsMenuCommandsHandler(ExplorerToolWindow parent)
        {
            _parentWindow = parent;
        }

        public void NewSubscription(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var menuInfo = menuItem.CommandParameter as MenuCommandParameters;
            if (menuInfo == null) return;
            ActivityLog.LogInformation("SQL Server Compact Toolbox", "Before loading Subscription window");
            OpenSubscriptionToolWindow(menuInfo);
        }

        public void DropSubscription(object sender, ExecutedRoutedEventArgs e)
        {
            if (MessageBox.Show("Do you really want to remove replication metadata from the SQL Server Compact database?", "SQL Server Compact Toolbox", MessageBoxButton.YesNo, MessageBoxImage.Warning, MessageBoxResult.No, MessageBoxOptions.None) == MessageBoxResult.No)
            {
                return;
            }

            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var menuInfo = menuItem.CommandParameter as MenuCommandParameters;

                if (menuInfo != null)
                {
                    try
                    {
                        SqlCeReplicationHelper.DropPublication(menuInfo.DatabaseInfo.ConnectionString, menuInfo.Name);
                        if (_parentWindow != null && _parentWindow.Content != null)
                        {
                            ExplorerControl control = _parentWindow.Content as ExplorerControl;
                            control.BuildDatabaseTree();
                        }
                        Helpers.DataConnectionHelper.LogUsage("SubscriptionDrop");
                    }
                    catch (Exception ex)
                    {
                        Helpers.DataConnectionHelper.SendError(ex, menuInfo.DatabaseInfo.DatabaseType, false);
                    }
                }
            }
        }



        private void OpenSubscriptionToolWindow(MenuCommandParameters menuInfo)
        {
            var pkg = _parentWindow.Package as SqlCeToolboxPackage;
            Debug.Assert(pkg != null, "Package property of the Explorere Tool Window should never be null, have you tried to create it manually and not through FindToolWindow()?");

            try
            {
                if (string.IsNullOrWhiteSpace(menuInfo.Name))
                {
                    menuInfo.Name = "Add";
                }
                var subsWindow = pkg.CreateWindow<SubscriptionWindow>(Math.Abs(menuInfo.DatabaseInfo.ConnectionString.GetHashCode() - menuInfo.Name.GetHashCode()));
                var control = subsWindow.Content as SubscriptionControl;
                control.DatabaseInfo = menuInfo.DatabaseInfo;
                if (menuInfo.MenuItemType == MenuType.Manage)
                {
                    control.Publication = menuInfo.Name;
                    control.IsNew = false;
                    subsWindow.Caption = menuInfo.Name;
                }
                else
                {
                    control.IsNew = true;
                    subsWindow.Caption = "New Subscription";
                }
                Helpers.DataConnectionHelper.LogUsage("SubscriptionManage");
            }
            catch (Exception ex)
            {
                Helpers.DataConnectionHelper.SendError(ex, menuInfo.DatabaseInfo.DatabaseType);
            }
            return;
        }

    }
}
