﻿using System.Windows;
using Microsoft.VisualStudio.PlatformUI;
using Microsoft.Windows.Controls.PropertyGrid;

namespace ErikEJ.SqlCeToolbox.Dialogs
{
    /// <summary>
    /// Interaction logic for OptionsDialog.xaml
    /// </summary>
    public partial class OptionsDialog : DialogWindow
    {
        private Helpers.UserOptions settings = new Helpers.UserOptions();

        public OptionsDialog()
        {
            InitializeComponent();
            this.Background = Helpers.VSThemes.GetWindowBackground();
        }

        private void close_Click(object sender, RoutedEventArgs e)
        {
            Properties.Settings.Default.DropTargetTables = settings.DropTargetTables;
            Properties.Settings.Default.ShowBinaryValuesInResult = settings.ShowBinaryValuesInResult;
            Properties.Settings.Default.ShowResultInGrid = settings.ShowResultInGrid;
            Properties.Settings.Default.DisplayDescriptionTable = settings.DisplayDescriptionTable;
            Properties.Settings.Default.IncludeSystemTablesInDocumentation = settings.IncludeSystemTablesInDocumentation;
            Properties.Settings.Default.DisplayObjectProperties = settings.DisplayObjectProperties;
            Properties.Settings.Default.MaxRowsToEdit = settings.MaxRowsToEdit;
            Properties.Settings.Default.IgnoreIdentityInInsertScript = settings.IgnoreIdentityInInsertScript;
            Properties.Settings.Default.KeepSchemaNames = settings.KeepServerSchemaNames;
            Properties.Settings.Default.ValidateConnectionsOnStart = settings.RemoveInvalidConnectionsOnStart;
            Properties.Settings.Default.MaxColumnWidth = settings.MaxColumnWidth;
            Properties.Settings.Default.Save();
            WriteQIKey(settings.ParticipateInQI);
            this.Close();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            settings.DropTargetTables = Properties.Settings.Default.DropTargetTables;
            settings.ShowBinaryValuesInResult = Properties.Settings.Default.ShowBinaryValuesInResult;
            settings.ShowResultInGrid = Properties.Settings.Default.ShowResultInGrid;
            settings.DisplayDescriptionTable = Properties.Settings.Default.DisplayDescriptionTable;
            settings.IncludeSystemTablesInDocumentation = Properties.Settings.Default.IncludeSystemTablesInDocumentation;
            settings.DisplayObjectProperties = Properties.Settings.Default.DisplayObjectProperties;
            settings.MaxRowsToEdit = Properties.Settings.Default.MaxRowsToEdit;
            settings.IgnoreIdentityInInsertScript = Properties.Settings.Default.IgnoreIdentityInInsertScript;
            settings.KeepServerSchemaNames = Properties.Settings.Default.KeepSchemaNames;
            settings.ParticipateInQI = ReadQIKey();
            settings.RemoveInvalidConnectionsOnStart = Properties.Settings.Default.ValidateConnectionsOnStart;
            settings.MaxColumnWidth = Properties.Settings.Default.MaxColumnWidth;
            Props.DisplaySummary = true;
            Props.SelectedObject = settings;
        }

//The registry key's name is SmartAssemblyReportUsage and it can take three values: True, False or Unknown: 
//•If the value is True, your program will report its usage. If the end user subsequently wishes to opt-out of feature usage reporting, set the value to False.
//•If the value is False, your program will not report its usage. If the end user subsequently wishes to opt-in to feature usage reporting, set the value to True. 
//•If the value is Unknown, or if the key does not exist, your program will ask the end user to decide whether to participate when the program next runs. The end user's decision will be recorded in this key.

        private const string keyName = @"HKEY_CURRENT_USER\Software\ErikEJ\SQL Server Compact Toolbox";
        private const string valueName = "SmartAssemblyReportUsage";
        private void WriteQIKey(bool participate)
        {
            Microsoft.Win32.Registry.SetValue(keyName, valueName, participate.ToString());
        }

        private bool ReadQIKey()
        {
            bool will = true;
            object participate = Microsoft.Win32.Registry.GetValue(keyName, valueName, "True");
            if (participate == null)
                return true;
            bool.TryParse(participate.ToString(), out will);
            return will;
        }

    }
}
