﻿using System;
using System.Collections.Generic;
using System.Data.Common;
using System.IO;
using System.Reflection;
using ErikEJ.SqlCeScripting;
using Microsoft.VisualStudio.Data;
using Microsoft.VisualStudio.Data.Interop;
using Microsoft.Win32;
using SmartAssembly.ReportException;
//using CodeSmith.Insight.Model;


namespace ErikEJ.SqlCeToolbox.Helpers
{

    internal class DataConnectionHelper
    {
        internal DataConnectionHelper()
        { }

        internal static Dictionary<string, DatabaseInfo> GetDataConnections(SqlCeToolboxPackage package, bool includeServerConnections, bool serverConnectionsOnly)
        {
            // http://www.mztools.com/articles/2007/MZ2007018.aspx
            Dictionary<string, DatabaseInfo> databaseList = new Dictionary<string, DatabaseInfo>();

            var objIVsDataConnectionsService = package.GetServiceHelper(typeof(IVsDataConnectionsService)) as IVsDataConnectionsService;
            var objIVsDataConnectionManager = package.GetServiceHelper(typeof(IVsDataConnectionManager)) as IVsDataConnectionManager;

            //Test code...
            //var objIVsDataProviderManager = package.GetServiceHelper(typeof(IVsDataProviderManager)) as IVsDataProviderManager;
            //var objIVsDataSourceManager = package.GetServiceHelper(typeof(IVsDataSourceManager)) as IVsDataSourceManager;

            //foreach (var objIVsDataSource in objIVsDataSourceManager.GetDataSources())
            //{
            //    System.Diagnostics.Debug.WriteLine(objIVsDataSource.DisplayName);
            //    System.Diagnostics.Debug.WriteLine(objIVsDataSource.Guid.ToString());

            //    foreach (var objProviderGuid in objIVsDataSource.GetProviders())
            //    {
            //        var objIVsDataProvider = objIVsDataProviderManager.GetDataProvider(objProviderGuid);
            //        System.Diagnostics.Debug.WriteLine(objIVsDataProvider.DisplayName);
            //        System.Diagnostics.Debug.WriteLine(objIVsDataProvider.Guid.ToString());
            //    }
            //}
            // End test code

            Guid provider35 = new Guid(Resources.SqlCompact35Provider);
            Guid provider40 = new Guid(Resources.SqlCompact40Provider);

            bool isV35Installed = IsV35Installed() && ProviderIsInstalled(provider35);
            bool isV40Installed = IsV40Installed() && ProviderIsInstalled(provider40);
            if (objIVsDataConnectionsService != null)
            {
                for (int i = 0; i < objIVsDataConnectionsService.Count; i++)
                {
                    try
                    {
                        var objProviderGuid = objIVsDataConnectionsService.GetProvider(i);
                        if (!serverConnectionsOnly)
                        {
                            if ((objProviderGuid == provider35 && isV35Installed) || (objProviderGuid == provider40 && isV40Installed) )
                            {
                                DatabaseType dbType = DatabaseType.SQLCE40;
                                if (objProviderGuid == provider35)
                                    dbType = DatabaseType.SQLCE35;

                                var sConnectionName = objIVsDataConnectionsService.GetConnectionName(i);
                                var sConnectionString = objIVsDataConnectionsService.GetConnectionString(i);
                                var objIVsDataConnection = objIVsDataConnectionManager.GetDataConnection(objProviderGuid, sConnectionString, true);
                                var conn = (DataConnection)objIVsDataConnection;
                                var adoConn = (DataConnectionSupport)conn.ConnectionSupport;
                                //Get the underlying connection string including password
                                var ceconn = (System.Data.Common.DbConnection)adoConn.ProviderObject;
                                if (!ceconn.Database.StartsWith("Mobile Device", StringComparison.OrdinalIgnoreCase))
                                {
                                    DatabaseInfo info = new DatabaseInfo();
                                    info.Caption = sConnectionName;
                                    info.FromServerExplorer = true;
                                    info.DatabaseType = dbType;
                                    info.ServerVersion = ceconn.ServerVersion;
                                    info.ConnectionString = ceconn.ConnectionString;
                                    if (!databaseList.ContainsKey(ceconn.ConnectionString))
                                        databaseList.Add(ceconn.ConnectionString, info);
                                }
                            }
                        }
                        if (includeServerConnections && objProviderGuid == new Guid(Resources.SqlServerDotNetProvider))
                        {
                            var sConnectionName = objIVsDataConnectionsService.GetConnectionName(i);
                            var sConnectionString = objIVsDataConnectionsService.GetConnectionString(i);
                            var objIVsDataConnection = objIVsDataConnectionManager.GetDataConnection(objProviderGuid, sConnectionString, true);
                            var conn = (DataConnection)objIVsDataConnection;
                            var adoConn = (DataConnectionSupport)conn.ConnectionSupport;
                            //Get the underlying connection string including password
                            var ceconn = (System.Data.Common.DbConnection)adoConn.ProviderObject;
                            DatabaseInfo info = new DatabaseInfo();
                            info.Caption = sConnectionName;
                            info.FromServerExplorer = true;
                            info.DatabaseType = DatabaseType.SQLServer;
                            info.ServerVersion = string.Empty;
                            info.ConnectionString = ceconn.ConnectionString;
                            if (!databaseList.ContainsKey(ceconn.ConnectionString))
                                databaseList.Add(ceconn.ConnectionString, info);
                        }
                    }
                    catch (KeyNotFoundException)
                    {
                        continue;
                    }
                    catch (NullReferenceException)
                    {
                        continue;
                    }
                }

            }
            return databaseList;
        }

        private static string GetPreferredInvariantName()
        {
            //Assume 3.5 is installed
            string invariant = Resources.SqlCompact35InvariantName;
            if (!IsV35Installed()) //So 4.0 is installed
            {
                invariant = Resources.SqlCompact40InvariantName;
            }
            return invariant;            
        }

        internal static Dictionary<string, DatabaseInfo> GetOwnDataConnections()
        {
            Dictionary<string, DatabaseInfo> databaseList = new Dictionary<string, DatabaseInfo>();

            //One of the runtimes must be installed
            if (!IsV35Installed() && !IsV40Installed())
                return databaseList;
            
            //Assumes 3.5 is installed
            string invariant = GetPreferredInvariantName();
            var factory = System.Data.Common.DbProviderFactories.GetFactory(invariant);
            using (var conn = factory.CreateConnection())
            {
                conn.ConnectionString = CreateStore(invariant);
                conn.Open();
                using (var cmd = factory.CreateCommand())
                {
                    cmd.Connection = conn;
                    cmd.CommandText = "SELECT FileName, Source, CeVersion FROM Databases";
                    var rdr = cmd.ExecuteReader();
                    while (rdr.Read())
                    {
                        string key = rdr[1].ToString();
                        DatabaseType type = (DatabaseType)int.Parse(rdr[2].ToString());
                        var info = new DatabaseInfo();
                        info.Caption = System.IO.Path.GetFileName(rdr[0].ToString());
                        info.DatabaseType = type;
                        info.FromServerExplorer = false;
                        info.ConnectionString = key;
                        info.ServerVersion = "4.0.0.0";
                        if (type == DatabaseType.SQLCE35)
                            info.ServerVersion = "3.5.1.0";
                        if (!databaseList.ContainsKey(key))
                        {
                            databaseList.Add(key, info);
                        }
                    }
                }
            }
            return databaseList;
        }

        internal static bool ProviderIsInstalled(Guid id)
        {
            var objIVsDataProviderManager = SqlCeToolboxPackage.GetGlobalService(typeof(IVsDataProviderManager)) as IVsDataProviderManager;
            var providerList = objIVsDataProviderManager.GetDataProviders();
            foreach (var provider in providerList)
            {
                if (provider.Guid == id)
                {
                    return true;
                }
            }
            return false;
        }

        internal void ValidateConnections(SqlCeToolboxPackage package)
        {
            var objIVsDataConnectionsService = package.GetServiceHelper(typeof(IVsDataConnectionsService)) as IVsDataConnectionsService;
            var objIVsDataConnectionManager = package.GetServiceHelper(typeof(IVsDataConnectionManager)) as IVsDataConnectionManager;

            var removals = new List<int>();

            if (objIVsDataConnectionsService != null)
            {
                for (int i = 0; i < objIVsDataConnectionsService.Count; i++)
                {
                    try
                    {
                        var objProviderGuid = objIVsDataConnectionsService.GetProvider(i);
                        if ((objProviderGuid == new Guid (Resources.SqlCompact35Provider) && IsV35Installed()) || (objProviderGuid == new Guid (Resources.SqlCompact40Provider) && IsV40Installed()))
                        {
                            var sConnectionName = objIVsDataConnectionsService.GetConnectionName(i);
                            var sConnectionString = objIVsDataConnectionsService.GetConnectionString(i);
                            var objIVsDataConnection = objIVsDataConnectionManager.GetDataConnection(objProviderGuid, sConnectionString, true);                            
                            var conn = (DataConnection)objIVsDataConnection;
                            conn.ConnectionSupport.Open(false);
                            if (conn != null)
                                conn.Close();
                            if (objIVsDataConnection != null)
                                objIVsDataConnection.Close();
                        }

                    }
                    catch (Exception ex)
                    {
                        if (ex.GetType().Name == "SqlCeException")
                        {
                            removals.Add(i);
                            continue;
                        }
                        else if (ex.GetType() == typeof(ArgumentException))
                        {
                            removals.Add(i);
                            continue;                            
                        }
                        else if (ex.GetType() == typeof(KeyNotFoundException))
                        {
                            removals.Add(i);
                            continue;                            
                        }
                        else
                        {
                            throw;
                        }
                    }

                }
                for (int i = removals.Count - 1; i >= 0; i--)
                {
                    try
                    {
                        objIVsDataConnectionsService.RemoveConnection(removals[i]);
                    }
                    catch (ArgumentException)
                    {
                        continue;
                    }
                    catch (IndexOutOfRangeException)
                    {
                        continue;
                    }
                    catch (KeyNotFoundException)
                    {
                        continue;
                    }
                }
            }

            var ownConnections = GetOwnDataConnections();
            foreach (var item in ownConnections)
            {
                try
                {
                    using (var test = CreateRepository(item.Value))
                    { }
                }
                catch (Exception ex)
                {
                    if (ex.GetType().Name == "SqlCeException")
                    {
                        System.Diagnostics.Debug.WriteLine(ex.Message);
                        RemoveDataConnection(item.Value.ConnectionString);
                        continue;
                    }
                    else
                    {
                        throw;
                    }
                }
            }
        }

        internal static void SaveDataConnection(SqlCeToolboxPackage package, string connectionString, string testString, string invariantName, Guid provider)
        {
            var objIVsDataConnectionsService = package.GetServiceHelper(typeof(IVsDataConnectionsService)) as IVsDataConnectionsService;
            if (objIVsDataConnectionsService != null)
            {
                var factory = System.Data.Common.DbProviderFactories.GetFactory(invariantName);
                var testConn = factory.CreateConnection();
                testConn.ConnectionString = testString;
                string filePath = testConn.Database;
                string savedName = Path.GetFileName(filePath);
                objIVsDataConnectionsService.AddConnection(savedName, provider, connectionString, true);
            }
        }

        internal static void SaveDataConnection(string connectionString, DatabaseType type)
        {
            var invariant = GetPreferredInvariantName();
            var factory = System.Data.Common.DbProviderFactories.GetFactory(invariant);
            var testConn = factory.CreateConnection();
            testConn.ConnectionString = connectionString;
            string filePath = testConn.Database;
            using (var conn = factory.CreateConnection())
            {
                conn.ConnectionString = CreateStore(invariant);
                conn.Open();
                using (var cmd = factory.CreateCommand())
                {
                    cmd.Connection = conn;
                    cmd.CommandText = "INSERT INTO Databases (Source, FileName, CeVersion) VALUES (@Source, @Filename, @CeVersion)";
                    
                    DbParameter param1 = factory.CreateParameter();
                    param1.DbType = System.Data.DbType.String;
                    param1.ParameterName = "@Source";
                    param1.Size = 2048;
                    param1.Value = connectionString;
                    cmd.Parameters.Add(param1);

                    DbParameter param2 = factory.CreateParameter();
                    param2.DbType = System.Data.DbType.String;
                    param2.ParameterName = "@Filename";
                    param2.Size = 512;
                    param2.Value = filePath;
                    cmd.Parameters.Add(param2);

                    DbParameter param3 = factory.CreateParameter();
                    param3.DbType = System.Data.DbType.Int32;
                    param3.ParameterName = "@CeVersion";
                    param3.Value = type.GetHashCode();
                    cmd.Parameters.Add(param3);

                    cmd.ExecuteNonQuery();
                }
            }
        }

        internal static void RemoveDataConnection(string connectionString)
        {
            var invariant = GetPreferredInvariantName();
            var factory = System.Data.Common.DbProviderFactories.GetFactory(invariant);
            using (var conn = factory.CreateConnection())
            {
                conn.ConnectionString = CreateStore(invariant);
                conn.Open();
                using (var cmd = factory.CreateCommand())
                {
                    cmd.Connection = conn;
                    cmd.CommandText = "DELETE FROM Databases WHERE Source = @Source;";
                    
                    DbParameter param1 = factory.CreateParameter();
                    param1.DbType = System.Data.DbType.String;
                    param1.ParameterName = "@Source";
                    param1.Size = 2048;
                    param1.Value = connectionString;
                    cmd.Parameters.Add(param1);

                    cmd.ExecuteNonQuery();
                }
            }
        }

        internal static void RemoveDataConnection(SqlCeToolboxPackage package, string connectionString, Guid provider)
        {
            var removals = new List<int>();
            var objIVsDataConnectionsService = package.GetServiceHelper(typeof(IVsDataConnectionsService)) as IVsDataConnectionsService;
            var objIVsDataConnectionManager = package.GetServiceHelper(typeof(IVsDataConnectionManager)) as IVsDataConnectionManager;
            if (objIVsDataConnectionsService != null)
            {
                for (int i = 0; i < objIVsDataConnectionsService.Count; i++)
                {
                    var objProviderGuid = objIVsDataConnectionsService.GetProvider(i);
                    if ((objProviderGuid == new Guid(Resources.SqlCompact35Provider)) || (objProviderGuid == new Guid(Resources.SqlCompact40Provider)))
                    {
                        var sConnectionString = objIVsDataConnectionsService.GetConnectionString(i);
                        var objIVsDataConnection = objIVsDataConnectionManager.GetDataConnection(objProviderGuid, sConnectionString, true);
                        if (objIVsDataConnection.DisplayConnectionString == connectionString)
                        {
                            removals.Add(i);
                        }
                    }
                }
            }

            for (int i = removals.Count - 1; i >= 0; i--)
            {
                try
                {
                    objIVsDataConnectionsService.RemoveConnection(removals[i]);
                }
                catch (ArgumentException)
                {
                    continue;
                }
                catch (IndexOutOfRangeException)
                {
                    continue;
                }
            }
        }

        // http://weblogs.asp.net/jgalloway/archive/2006/10/29/Showing-a-Connection-String-prompt-in-a-WinForm-application.aspx
        public static string PromptForConnectionString(string currentConnectionString = "")
        {
            MSDASC.DataLinks dataLinks = new MSDASC.DataLinks();
            ADODB.Connection dialogConnection;
            string generatedConnectionString = string.Empty;

            if (currentConnectionString == String.Empty)
            {
                dialogConnection = (ADODB.Connection)dataLinks.PromptNew();
                generatedConnectionString = dialogConnection.ConnectionString.ToString();
            }
            else
            {
                dialogConnection = new ADODB.Connection();
                dialogConnection.Provider = "SQLOLEDB.1";
                ADODB.Property persistProperty = dialogConnection.Properties["Persist Security Info"];
                persistProperty.Value = true;

                dialogConnection.ConnectionString = currentConnectionString;
                dataLinks = new MSDASC.DataLinks();

                object objConn = dialogConnection;
                if (dataLinks.PromptEdit(ref objConn))
                {
                    generatedConnectionString = dialogConnection.ConnectionString.ToString();
                }
            }
            generatedConnectionString = generatedConnectionString.Replace("Provider=SQLOLEDB.1;", string.Empty);
            if (
                    !generatedConnectionString.Contains("Integrated Security=SSPI")
                    && !generatedConnectionString.Contains("Trusted_Connection=True")
                    && !generatedConnectionString.Contains("Password=")
                    && !generatedConnectionString.Contains("Pwd=")
                )
                if (dialogConnection.Properties["Password"] != null && dialogConnection.Properties["Password"].Value != null)
                    generatedConnectionString += ";Password=" + dialogConnection.Properties["Password"].Value.ToString();

            return generatedConnectionString;
        }

        private static string CreateStore(string invariant)
        {
            var factory = System.Data.Common.DbProviderFactories.GetFactory(invariant);
            string fileName = GetSdfName(invariant);
            string connString = string.Format("Data Source={0};", fileName);
            bool created = false;
            if (!File.Exists(fileName))
            {
                var sdf = Resources.SqlCe35AddinStore;
                if (invariant == Resources.SqlCompact40InvariantName)
                    sdf = Resources.SqlCe40AddinStore;
                using (Stream stream = new MemoryStream(sdf))
                {
                    // Create a FileStream object to write a stream to a file 
                    using (FileStream fileStream = File.Create(fileName, (int)stream.Length))
                    {
                        // Fill the bytes[] array with the stream data 
                        byte[] bytesInStream = new byte[stream.Length];
                        stream.Read(bytesInStream, 0, (int)bytesInStream.Length);
                        // Use FileStream object to write to the specified file 
                        fileStream.Write(bytesInStream, 0, bytesInStream.Length);
                        created = true;
                    }
                }
            }

            using (var conn = factory.CreateConnection())
            {
                if (created)
                {
                    conn.ConnectionString = connString;
                    conn.Open();
                    using (var cmd = factory.CreateCommand())
                    {
                        cmd.Connection = conn;
                        cmd.CommandText = "CREATE TABLE Databases (Id INT IDENTITY, Source nvarchar(2048) NOT NULL, FileName nvarchar(512) NOT NULL, CeVersion int NOT NULL)";
                        cmd.ExecuteNonQuery();
                    }
                }
            }
            return connString;
        }

        private static string GetSdfName(string invariant)
        {
            string file = "SqlCe35AddinStore.sdf";
            if (invariant == Resources.SqlCompact40InvariantName)
                file = "SqlCe40AddinStore.sdf";
            string fileName = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), file);
            return fileName;
        }

        public static IGenerator CreateGenerator(IRepository repository, string outFile, DatabaseType databaseType)
        {
            switch (databaseType)
            {
                case DatabaseType.SQLServer:
                    return new Generator(repository, outFile, false, false, false, Properties.Settings.Default.KeepSchemaNames);
                case DatabaseType.SQLCE35:
                    return new Generator(repository, outFile);
                case DatabaseType.SQLCE40:
                    return new Generator4(repository, outFile);
                default:
                    return null;
            }
        }

        public static ISqlCeHelper CreateSqlCeHelper(DatabaseType databaseType)
        {
            switch (databaseType)
            {
                case DatabaseType.SQLServer:
                case DatabaseType.SQLCE35:
                    return new SqlCeHelper();
                case DatabaseType.SQLCE40:
                    return new SqlCeHelper4();
                default:
                    return null;
            }
        }

        public static IGenerator CreateGenerator(IRepository repository, DatabaseType databaseType)
        {
            return CreateGenerator(repository, null, databaseType);
        }

        public static IRepository CreateRepository(DatabaseInfo databaseInfo)
        {
            switch (databaseInfo.DatabaseType)
            {
                case DatabaseType.SQLCE35:
                    return new DBRepository(databaseInfo.ConnectionString);
                case DatabaseType.SQLCE40:
                    return new DB4Repository(databaseInfo.ConnectionString);
                case DatabaseType.SQLServer:
                    return new ServerDBRepository(databaseInfo.ConnectionString, Properties.Settings.Default.KeepSchemaNames);
                default:
                    return null;
            }
        }

        public static bool IsPremiumOrUltimate()
        {
            // From http://blogs.msdn.com/b/heaths/archive/2010/05/04/detection-keys-for-net-framework-4-0-and-visual-studio-2010.aspx
            //Key HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\DevDiv\VS\Servicing\10.0\$(var.ProductEdition)\$(var.LCID) 
            //The values for $(var.ProductEdition) include the following table. 
            //Visual Studio 2010 Ultimate VSTSCore 
            //Visual Studio 2010 Premium VSTDCore 
            //Visual Studio 2010 Professional PROCore 
            //Visual Studio 2010 Shell (Integrated) IntShell 
            using (var key = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry32))
            {
                var ultimateKey = key.OpenSubKey(@"SOFTWARE\Microsoft\DevDiv\VS\Servicing\10.0\VSTSCore");
                if (ultimateKey != null)
                    return true;

                var premiumKey = key.OpenSubKey(@"SOFTWARE\Microsoft\DevDiv\VS\Servicing\10.0\VSTDCore");
                if (premiumKey != null)
                    return true;
            }
            return false;
        }

        public static bool CheckVersion(string lookingFor)
        {
            try
            {
                using (var wc = new System.Net.WebClient())
                {
                    wc.Proxy = System.Net.WebRequest.GetSystemWebProxy();
                    var xDoc = new System.Xml.XmlDocument();
                    string s = wc.DownloadString(@"http://www.sqlcompact.dk/SqlCeToolboxVersions.xml");
                    xDoc.LoadXml(s);

                    string newVersion = xDoc.DocumentElement.Attributes[lookingFor].Value;
                    
                    Version vN = new Version(newVersion );
                    if (vN > Assembly.GetExecutingAssembly().GetName().Version)
                    {
                        return true;
                    }

                }
            }
            catch { }
            return false;
        }

        internal static void LogUsage(string feature)
        {
            SmartAssembly.ReportUsage.UsageCounter.ReportUsage(feature);
        }

        internal static string SendError(Exception ex, DatabaseType dbType, bool report = true)
        {
            if (report)
            {
                ExceptionReporting.Report(ex);
            }
            else
            { 
                System.Windows.MessageBox.Show(Helpers.DataConnectionHelper.CreateSqlCeHelper(dbType).FormatError(ex), "SQL Server Compact Toolbox");
            }
            return string.Empty; //CreateSqlCeHelper(dbType).FormatError(ex);
        }

        internal static bool IsV40Installed()
        {
            return new SqlCeHelper4().IsV40Installed();
        }

        internal static bool IsV35Installed()
        {
            return new SqlCeHelper4().IsV35Installed();
        }

        internal static bool IsSyncFx21Installed()
        {
            try
            {
                System.Reflection.Assembly.Load("Microsoft.Synchronization.Data.SqlServerCe, Version=3.1.0.0, Culture=neutral, PublicKeyToken=89845dcd8080cc91");
            }
            catch
            {
                return false;
            }
            return true;
        }

    }
}
