﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using EnvDTE;
using Microsoft.Windows.Controls;
using Microsoft.Win32;

namespace ErikEJ.SqlCeToolbox.Helpers
{
    internal class EnvDTEHelper
    {
        public Project GetProject(DTE dte)
        {
            foreach (SelectedItem item in dte.SelectedItems)
            {
                if (item.Project != null) return item.Project;
                if (item.ProjectItem != null) return item.ProjectItem.ContainingProject;
            }
            return null;
        }

        public ProjectItem GetProjectConfig(Project project)
        {
            foreach (ProjectItem item in project.ProjectItems)
            {
                if (item.Name.ToLowerInvariant() == "app.config")
                {
                    return item;
                }
            }
            return null;
        }

        public ProjectItem GetProjectEdmx(Project project, string model)
        {
            foreach (ProjectItem item in project.ProjectItems)
            {
                if (item.Name.ToLowerInvariant() == model.ToLowerInvariant() + ".edmx")
                {
                    return item;
                }
            }
            return null;
        }

        public ProjectItem GetProjectDC(Project project, string model, string extension)
        {
            foreach (ProjectItem item in project.ProjectItems)
            {
                if (item.Name.ToLowerInvariant() == model.ToLowerInvariant() + extension)
                {
                    return item;
                }
            }
            return null;
        }

        public ProjectItem GetProjectDataContextClass(Project project, string classFileName)
        {
            foreach (ProjectItem item in project.ProjectItems)
            {
                if (item.Name.ToLowerInvariant() == classFileName)
                {
                    return item;
                }
            }
            return null;
        }

        public static void AddReference(Project project, string reference)
        {
            VSLangProj.VSProject vsProject = (VSLangProj.VSProject)project.Object;
            vsProject.References.Add(reference);
        }

        public bool ContainsEFSqlCeReference(Project project)
        {
            VSLangProj.VSProject vsProject = (VSLangProj.VSProject)project.Object;
            for (int i = 1; i < vsProject.References.Count + 1; i++)
            {
                if (vsProject.References.Item(i).Name == "EntityFramework.SqlServerCompact"
                    && new Version(vsProject.References.Item(i).Version) >= new Version(6, 0, 0, 0))
                    return true;
            }
            return false;
        }

        public List<Guid> AllowedWPProjectKinds
        {
            get
            {
                return new List<Guid>() 
                {
                    new Guid("{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}"),
                    new Guid("{F184B08F-C81C-45F6-A57F-5ABD9991F28F}")
                };
            }
        }

        public static Guid VBProject = new Guid("{F184B08F-C81C-45F6-A57F-5ABD9991F28F}");

        public List<Guid> AllowedProjectKinds
        {
            // http://www.mztools.com/Articles/2008/MZ2008017.aspx

            get
            {
                return new List<Guid>() 
                {
                    //Windows (C#) {FAE04EC0-301F-11D3-BF4B-00C04F79EFBC} 
                    new Guid("{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}"),
                    //Windows (VB.NET) {F184B08F-C81C-45F6-A57F-5ABD9991F28F} 
                    new Guid("{F184B08F-C81C-45F6-A57F-5ABD9991F28F}"),
                    //Web Application {349C5851-65DF-11DA-9384-00065B846F21} 
                    new Guid("{349C5851-65DF-11DA-9384-00065B846F21}"),
                    //Web Site {E24C65DC-7377-472B-9ABA-BC803B73C61A} 
                    new Guid("{E24C65DC-7377-472B-9ABA-BC803B73C61A}"),
                    //Windows Communication Foundation (WCF) {3D9AD99F-2412-4246-B90B-4EAA41C64699} 
                    new Guid("{3D9AD99F-2412-4246-B90B-4EAA41C64699}"),
                    //Windows Presentation Foundation (WPF) {60DC8134-EBA5-43B8-BCC9-BB4BC16C2548} 
                    new Guid("{60DC8134-EBA5-43B8-BCC9-BB4BC16C2548}")                    
                };
            }
        }

        public List<Guid> WebProjectKinds
        {
            get
            {
                return new List<Guid>() 
                {
                    new Guid("{349C5851-65DF-11DA-9384-00065B846F21}"),
                    new Guid("{E24C65DC-7377-472B-9ABA-BC803B73C61A}")                    
                };
            }
        }


        /// <summary>
        /// Gets the installation directory of a given Visual Studio Version
        /// </summary>
        /// <param name="version">Visual Studio Version</param>
        /// <returns>Null if not installed the installation directory otherwise</returns>
        internal string GetVisualStudioInstallationDir()
        {
            string registryKeyString = String.Format(@"SOFTWARE{0}Microsoft\VisualStudio\{1}",
                Environment.Is64BitProcess ? @"\Wow6432Node\" : @"\",
                "12.0");

            using (RegistryKey localMachineKey = Registry.LocalMachine.OpenSubKey(registryKeyString))
            {
                return localMachineKey.GetValue("InstallDir") as string;
            }
        }
    }
}
