﻿using System.Windows;
using System.Windows.Controls;

namespace ErikEJ.SqlCeToolbox.Helpers
{
    public class DatabaseTreeViewItem : TreeViewItem
    {
        public string MetaData { get; set; }

        public override string ToString()
        {
            return MetaData;
        }
    
    }

    public sealed class TreeViewHelper
    {
        private TreeViewHelper()
        { }

        public static DatabaseTreeViewItem CreateTreeViewItemWithImage(string name, string imageName, bool showExpander)
        {
            return CreateTreeViewItemWithImageAndTooltip(name, imageName, showExpander, null);
        }

        public static DatabaseTreeViewItem CreateTreeViewItemWithImageAndTooltip(string name, string imageName, bool showExpander, string toolTip)
        {
            var stackpanel = new StackPanel { Orientation = Orientation.Horizontal, Margin = new System.Windows.Thickness(2) };
            stackpanel.Children.Add(ImageHelper.GetImageFromResource(imageName));
            stackpanel.Children.Add(new TextBlock { Text = " " + name, Foreground = Helpers.VSThemes.GetWindowText() });

            var databaseTreeViewItem = new DatabaseTreeViewItem { Header = stackpanel, MetaData = name };
            databaseTreeViewItem.MouseRightButtonDown += DatabaseTreeViewItemMouseRightButtonDown;
            databaseTreeViewItem.ContextMenu = new ContextMenu { Visibility = Visibility.Hidden };
            if (!string.IsNullOrWhiteSpace(toolTip))
            {
                databaseTreeViewItem.ToolTip = toolTip;
            }
            if (showExpander) databaseTreeViewItem.Items.Add("Loading...");
            return databaseTreeViewItem;
        }

        static void DatabaseTreeViewItemMouseRightButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            ((TreeViewItem)sender).IsSelected = true;
            e.Handled = true;
        }

        public static TreeViewItem GetTypesItem(TreeViewItem viewItem)
        {
            var types = CreateTreeViewItemWithImage("Data Types", "../Resources/folder.png", false);

            var numbersItem = CreateTreeViewItemWithImage("Exact Numerics", "../Resources/folder.png", false);
            numbersItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("bit", "../Resources/sp.png", false, SqlCeToolbox.Resources.bit));
            numbersItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("tinyint", "../Resources/sp.png", false, SqlCeToolbox.Resources.tinyint));
            numbersItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("smallint", "../Resources/sp.png", false, SqlCeToolbox.Resources.smallint));
            numbersItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("int", "../Resources/sp.png", false, SqlCeToolbox.Resources.integer));
            numbersItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("bigint", "../Resources/sp.png", false, SqlCeToolbox.Resources.bigint));
            numbersItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("numeric", "../Resources/sp.png", false, SqlCeToolbox.Resources.numeric));
            numbersItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("money", "../Resources/sp.png", false, SqlCeToolbox.Resources.money));
            types.Items.Add(numbersItem);

            var floatItem = CreateTreeViewItemWithImage("Approximate Numerics", "../Resources/folder.png", false);
            floatItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("float", "../Resources/sp.png", false, SqlCeToolbox.Resources.floating));
            floatItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("real", "../Resources/sp.png", false, SqlCeToolbox.Resources.real));
            types.Items.Add(floatItem);

            var dateItem = CreateTreeViewItemWithImage("Date and Time", "../Resources/folder.png", false);
            dateItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("datetime", "../Resources/sp.png", false, SqlCeToolbox.Resources.datetime));
            types.Items.Add(dateItem);

            var stringItem = CreateTreeViewItemWithImage("Unicode Character Strings", "../Resources/folder.png", false);
            stringItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("nchar", "../Resources/sp.png", false, SqlCeToolbox.Resources.nchar));
            stringItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("nvarchar", "../Resources/sp.png", false, SqlCeToolbox.Resources.nvarchar));
            stringItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("ntext", "../Resources/sp.png", false, SqlCeToolbox.Resources.ntext));
            types.Items.Add(stringItem);

            var binaryItem = CreateTreeViewItemWithImage("Binary Values", "../Resources/folder.png", false);
            binaryItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("binary", "../Resources/sp.png", false, SqlCeToolbox.Resources.binary));
            binaryItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("varbinary", "../Resources/sp.png", false, SqlCeToolbox.Resources.varbinary));
            binaryItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("image", "../Resources/sp.png", false, SqlCeToolbox.Resources.image));
            types.Items.Add(binaryItem);

            var otherItem = CreateTreeViewItemWithImage("Other Data Types", "../Resources/folder.png", false);
            otherItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("rowversion", "../Resources/sp.png", false, SqlCeToolbox.Resources.rowversion));
            otherItem.Items.Add(CreateTreeViewItemWithImageAndTooltip("uniqueidentifier", "../Resources/sp.png", false, SqlCeToolbox.Resources.uniqueidentifier));
            types.Items.Add(otherItem);

            return types;
        }

    }
}